@@ -43,7 +43,7 @@ const double MacroAssembler::twoToThe32 = (double)0x100000000ull;
 
 void MacroAssembler::jitAssert(const ScopedLambda<Jump(void)>& functor)
 {
-    if (Options::enableJITDebugAssetions()) {
+    if (Options::enableJITDebugAssertions()) {
         Jump passed = functor();
         breakpoint();
         passed.link(this);
@@ -53,13 +53,13 @@ void MacroAssembler::jitAssert(const ScopedLambda<Jump(void)>& functor)
 #if ENABLE(MASM_PROBE)
 static void stdFunctionCallback(Probe::Context& context)
 {
-    auto func = context.arg<const std::function<void(Probe::Context&)>*>();
+    auto func = context.arg<const Function<void(Probe::Context&)>*>();
     (*func)(context);
 }
     
-void MacroAssembler::probe(std::function<void(Probe::Context&)> func)
+void MacroAssembler::probe(Function<void(Probe::Context&)> func)
 {
-    probe(stdFunctionCallback, new std::function<void(Probe::Context&)>(func));
+    probe(stdFunctionCallback, new Function<void(Probe::Context&)>(WTFMove(func)));
 }
 #endif // ENABLE(MASM_PROBE)
 
