@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016-2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2016-2018 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -172,6 +172,7 @@ void fixSSAGlobally(Procedure& proc)
     TimingScope timingScope("fixSSA: convert");
     InsertionSet insertionSet(proc);
     IndexSparseSet<KeyValuePair<unsigned, Value*>> mapping(proc.variables().size());
+    IndexSet<Value*> valuesToDelete;
     for (BasicBlock* block : proc.blocksInPreOrder()) {
         mapping.clear();
         
@@ -207,6 +208,7 @@ void fixSSAGlobally(Procedure& proc)
                 Variable* variable = variableValue->variable();
 
                 value->replaceWithIdentity(ensureMapping(variable, valueIndex, value->origin()));
+                valuesToDelete.add(value);
                 break;
             }
                 
@@ -240,12 +242,25 @@ void fixSSAGlobally(Procedure& proc)
                 }
                 
                 insertionSet.insert<UpsilonValue>(
-                    upsilonInsertionPoint, upsilonOrigin, mappedValue, phi);
+                    upsilonInsertionPoint, upsilonOrigin, mappedValue->foldIdentity(), phi);
             }
         }
 
         insertionSet.execute(block);
     }
+    
+    // This is isn't strictly necessary, but it leaves the IR nice and tidy, which is particularly
+    // useful for phases that do size estimates.
+    for (BasicBlock* block : proc) {
+        block->values().removeAllMatching(
+            [&] (Value* value) -> bool {
+                if (!valuesToDelete.contains(value) && value->opcode() != Nop)
+                    return false;
+                
+                proc.deleteValue(value);
+                return true;
+            });
+    }
 
     if (B3FixSSAInternal::verbose) {
         dataLog("B3 after SSA conversion:\n");
@@ -285,6 +300,17 @@ void demoteValues(Procedure& proc, const IndexSet<Value*>& values)
     // Change accesses to the values to accesses to the stack slots.
     InsertionSet insertionSet(proc);
     for (BasicBlock* block : proc) {
+        if (block->numPredecessors()) {
+            // Deal with terminals that produce values (i.e. patchpoint terminals, like the ones we
+            // generate for the allocation fast path).
+            Value* value = block->predecessor(0)->last();
+            Variable* variable = map.get(value);
+            if (variable) {
+                RELEASE_ASSERT(block->numPredecessors() == 1); // Critical edges better be broken.
+                insertionSet.insert<VariableValue>(0, Set, value->origin(), variable, value);
+            }
+        }
+        
         for (unsigned valueIndex = 0; valueIndex < block->size(); ++valueIndex) {
             Value* value = block->at(valueIndex);
 
@@ -312,8 +338,10 @@ void demoteValues(Procedure& proc, const IndexSet<Value*>& values)
             }
 
             if (Variable* variable = map.get(value)) {
-                insertionSet.insert<VariableValue>(
-                    valueIndex + 1, Set, value->origin(), variable, value);
+                if (valueIndex + 1 < block->size()) {
+                    insertionSet.insert<VariableValue>(
+                        valueIndex + 1, Set, value->origin(), variable, value);
+                }
             }
         }
         insertionSet.execute(block);
@@ -324,6 +352,9 @@ bool fixSSA(Procedure& proc)
 {
     PhaseScope phaseScope(proc, "fixSSA");
 
+    if (proc.variables().isEmpty())
+        return false;
+    
     // Lots of variables have trivial local liveness. We can allocate those without any
     // trouble.
     fixSSALocally(proc);
@@ -336,7 +367,6 @@ bool fixSSA(Procedure& proc)
     if (proc.variables().isEmpty())
         return false;
     
-    // We know that we have variables to optimize, so do that now.
     breakCriticalEdges(proc);
 
     fixSSAGlobally(proc);
