@@ -466,7 +466,7 @@ protected:
             return m_value & 0xffffffff;
         }
 
-        bool operator==(const InterferenceEdge other) const
+        bool operator==(const InterferenceEdge& other) const
         {
             return m_value == other.m_value;
         }
@@ -1673,7 +1673,7 @@ protected:
         m_worklistMoves.startAddingLowPriorityMoves();
         for (BasicBlock* block : m_code) {
             for (Inst& inst : *block) {
-                if (std::optional<unsigned> defArgIndex = inst.shouldTryAliasingDef()) {
+                if (Optional<unsigned> defArgIndex = inst.shouldTryAliasingDef()) {
                     Arg op1 = inst.args[*defArgIndex - 2];
                     Arg op2 = inst.args[*defArgIndex - 1];
                     Arg dest = inst.args[*defArgIndex];
