@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2013-2018 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,6 +28,7 @@
 #include "CompilationResult.h"
 #include "DFGCompilationKey.h"
 #include "DFGCompilationMode.h"
+#include "DFGDesiredGlobalProperties.h"
 #include "DFGDesiredIdentifiers.h"
 #include "DFGDesiredTransitions.h"
 #include "DFGDesiredWatchpoints.h"
@@ -36,6 +37,7 @@
 #include "DeferredCompilationCallback.h"
 #include "Operands.h"
 #include "ProfilerCompilation.h"
+#include "RecordedStatuses.h"
 #include <wtf/HashMap.h>
 #include <wtf/ThreadSafeRefCounted.h>
 
@@ -50,11 +52,12 @@ class ThreadData;
 
 #if ENABLE(DFG_JIT)
 
-struct Plan : public ThreadSafeRefCounted<Plan> {
+class Plan : public ThreadSafeRefCounted<Plan> {
+public:
     Plan(
         CodeBlock* codeBlockToCompile, CodeBlock* profiledDFGCodeBlock,
         CompilationMode, unsigned osrEntryBytecodeIndex,
-        const Operands<JSValue>& mustHandleValues);
+        const Operands<Optional<JSValue>>& mustHandleValues);
     ~Plan();
 
     void compileInThread(ThreadData*);
@@ -67,53 +70,50 @@ struct Plan : public ThreadSafeRefCounted<Plan> {
     
     CompilationKey key();
     
-    void markCodeBlocks(SlotVisitor&);
     template<typename Func>
     void iterateCodeBlocksForGC(const Func&);
     void checkLivenessAndVisitChildren(SlotVisitor&);
     bool isKnownToBeLiveDuringGC();
+    void finalizeInGC();
     void cancel();
 
-    bool canTierUpAndOSREnter() const { return !tierUpAndOSREnterBytecodes.isEmpty(); }
-    
+    bool canTierUpAndOSREnter() const { return !m_tierUpAndOSREnterBytecodes.isEmpty(); }
+
     void cleanMustHandleValuesIfNecessary();
-    
-    // Warning: pretty much all of the pointer fields in this object get nulled by cancel(). So, if
-    // you're writing code that is callable on the cancel path, be sure to null check everything!
-    
-    VM* vm;
 
-    // These can be raw pointers because we visit them during every GC in checkLivenessAndVisitChildren.
-    CodeBlock* codeBlock;
-    CodeBlock* profiledDFGCodeBlock;
+    VM* vm() const { return m_vm; }
 
-    CompilationMode mode;
-    const unsigned osrEntryBytecodeIndex;
-    Operands<JSValue> mustHandleValues;
-    bool mustHandleValuesMayIncludeGarbage { true };
-    Lock mustHandleValueCleaningLock;
-    
-    ThreadData* threadData;
+    CodeBlock* codeBlock() { return m_codeBlock; }
 
-    RefPtr<Profiler::Compilation> compilation;
+    bool isFTL() const { return DFG::isFTL(m_mode); }
+    CompilationMode mode() const { return m_mode; }
+    unsigned osrEntryBytecodeIndex() const { return m_osrEntryBytecodeIndex; }
+    const Operands<Optional<JSValue>>& mustHandleValues() const { return m_mustHandleValues; }
+    ThreadData* threadData() const { return m_threadData; }
+    Profiler::Compilation* compilation() const { return m_compilation.get(); }
 
-    std::unique_ptr<Finalizer> finalizer;
-    
-    RefPtr<InlineCallFrameSet> inlineCallFrames;
-    DesiredWatchpoints watchpoints;
-    DesiredIdentifiers identifiers;
-    DesiredWeakReferences weakReferences;
-    DesiredTransitions transitions;
-    
-    bool willTryToTierUp { false };
+    Finalizer* finalizer() const { return m_finalizer.get(); }
+    void setFinalizer(std::unique_ptr<Finalizer>&& finalizer) { m_finalizer = WTFMove(finalizer); }
 
-    HashMap<unsigned, Vector<unsigned>> tierUpInLoopHierarchy;
-    Vector<unsigned> tierUpAndOSREnterBytecodes;
+    RefPtr<InlineCallFrameSet> inlineCallFrames() const { return m_inlineCallFrames; }
+    DesiredWatchpoints& watchpoints() { return m_watchpoints; }
+    DesiredIdentifiers& identifiers() { return m_identifiers; }
+    DesiredWeakReferences& weakReferences() { return m_weakReferences; }
+    DesiredTransitions& transitions() { return m_transitions; }
+    DesiredGlobalProperties& globalProperties() { return m_globalProperties; }
+    RecordedStatuses& recordedStatuses() { return m_recordedStatuses; }
+
+    bool willTryToTierUp() const { return m_willTryToTierUp; }
+    void setWillTryToTierUp(bool willTryToTierUp) { m_willTryToTierUp = willTryToTierUp; }
+
+    HashMap<unsigned, Vector<unsigned>>& tierUpInLoopHierarchy() { return m_tierUpInLoopHierarchy; }
+    Vector<unsigned>& tierUpAndOSREnterBytecodes() { return m_tierUpAndOSREnterBytecodes; }
 
     enum Stage { Preparing, Compiling, Ready, Cancelled };
-    Stage stage;
+    Stage stage() const { return m_stage; }
 
-    RefPtr<DeferredCompilationCallback> callback;
+    DeferredCompilationCallback* callback() const { return m_callback.get(); }
+    void setCallback(Ref<DeferredCompilationCallback>&& callback) { m_callback = WTFMove(callback); }
 
 private:
     bool computeCompileTimes() const;
@@ -122,9 +122,48 @@ private:
     enum CompilationPath { FailPath, DFGPath, FTLPath, CancelPath };
     CompilationPath compileInThreadImpl();
     
+    bool isStillValidOnMainThread();
     bool isStillValid();
     void reallyAdd(CommonData*);
 
+    // Warning: pretty much all of the pointer fields in this object get nulled by cancel(). So, if
+    // you're writing code that is callable on the cancel path, be sure to null check everything!
+
+    VM* m_vm;
+
+    // These can be raw pointers because we visit them during every GC in checkLivenessAndVisitChildren.
+    CodeBlock* m_codeBlock;
+    CodeBlock* m_profiledDFGCodeBlock;
+
+    CompilationMode m_mode;
+    const unsigned m_osrEntryBytecodeIndex;
+    Operands<Optional<JSValue>> m_mustHandleValues;
+    bool m_mustHandleValuesMayIncludeGarbage { true };
+    Lock m_mustHandleValueCleaningLock;
+
+    ThreadData* m_threadData;
+
+    RefPtr<Profiler::Compilation> m_compilation;
+
+    std::unique_ptr<Finalizer> m_finalizer;
+
+    RefPtr<InlineCallFrameSet> m_inlineCallFrames;
+    DesiredWatchpoints m_watchpoints;
+    DesiredIdentifiers m_identifiers;
+    DesiredWeakReferences m_weakReferences;
+    DesiredTransitions m_transitions;
+    DesiredGlobalProperties m_globalProperties;
+    RecordedStatuses m_recordedStatuses;
+
+    bool m_willTryToTierUp { false };
+
+    HashMap<unsigned, Vector<unsigned>> m_tierUpInLoopHierarchy;
+    Vector<unsigned> m_tierUpAndOSREnterBytecodes;
+
+    Stage m_stage;
+
+    RefPtr<DeferredCompilationCallback> m_callback;
+
     MonotonicTime m_timeBeforeFTL;
 };
 
