@@ -73,7 +73,7 @@ inline void ConservativeRoots::genericAddPointer(void* p, HeapVersion markingVer
     HeapUtil::findGCObjectPointersForMarking(
         m_heap, markingVersion, newlyAllocatedVersion, filter, p,
         [&] (void* p, HeapCell::Kind cellKind) {
-            if (cellKind == HeapCell::JSCell)
+            if (isJSCellKind(cellKind))
                 markHook.markKnownJSCell(static_cast<JSCell*>(p));
             
             if (m_size == m_capacity)
