@@ -79,7 +79,7 @@ void RemoteInspector::start()
             if (GRefPtr<GDBusConnection> connection = adoptGRef(g_dbus_connection_new_for_address_finish(result, &error.outPtr())))
                 inspector->setupConnection(WTFMove(connection));
             else if (!g_error_matches(error.get(), G_IO_ERROR, G_IO_ERROR_CANCELLED))
-                WTFLogAlways("RemoteInspector failed to connect to inspector server at: %s: %s", g_getenv("WEBKIT_INSPECTOR_SERVER"), error->message);
+                g_warning("RemoteInspector failed to connect to inspector server at: %s: %s", g_getenv("WEBKIT_INSPECTOR_SERVER"), error->message);
     }, this);
 }
 
@@ -178,7 +178,7 @@ static void dbusConnectionCallAsyncReadyCallback(GObject* source, GAsyncResult*
     GUniqueOutPtr<GError> error;
     GRefPtr<GVariant> resultVariant = adoptGRef(g_dbus_connection_call_finish(G_DBUS_CONNECTION(source), result, &error.outPtr()));
     if (!resultVariant && !g_error_matches(error.get(), G_IO_ERROR, G_IO_ERROR_CANCELLED))
-        WTFLogAlways("RemoteInspector failed to send DBus message: %s", error->message);
+        g_warning("RemoteInspector failed to send DBus message: %s", error->message);
 }
 
 TargetListing RemoteInspector::listingForInspectionTarget(const RemoteInspectionTarget& target) const
@@ -293,7 +293,6 @@ void RemoteInspector::receivedGetTargetListMessage()
 
 void RemoteInspector::receivedSetupMessage(unsigned targetIdentifier)
 {
-    std::lock_guard<Lock> lock(m_mutex);
     setup(targetIdentifier);
 }
 
@@ -328,9 +327,13 @@ void RemoteInspector::receivedCloseMessage(unsigned targetIdentifier)
 
 void RemoteInspector::setup(unsigned targetIdentifier)
 {
-    RemoteControllableTarget* target = m_targetMap.get(targetIdentifier);
-    if (!target)
-        return;
+    RemoteControllableTarget* target;
+    {
+        std::lock_guard<Lock> lock(m_mutex);
+        target = m_targetMap.get(targetIdentifier);
+        if (!target)
+            return;
+    }
 
     auto connectionToTarget = adoptRef(*new RemoteConnectionToTarget(*target));
     ASSERT(is<RemoteInspectionTarget>(target) || is<RemoteAutomationTarget>(target));
@@ -338,6 +341,8 @@ void RemoteInspector::setup(unsigned targetIdentifier)
         connectionToTarget->close();
         return;
     }
+
+    std::lock_guard<Lock> lock(m_mutex);
     m_targetConnectionMap.set(targetIdentifier, WTFMove(connectionToTarget));
 
     updateHasActiveDebugSession();
@@ -349,7 +354,7 @@ void RemoteInspector::sendMessageToTarget(unsigned targetIdentifier, const char*
         connectionToTarget->sendMessageToTarget(String::fromUTF8(message));
 }
 
-void RemoteInspector::requestAutomationSession(const char* sessionID)
+void RemoteInspector::requestAutomationSession(const char* sessionID, const Client::SessionCapabilities& capabilities)
 {
     if (!m_client)
         return;
@@ -360,7 +365,7 @@ void RemoteInspector::requestAutomationSession(const char* sessionID)
     if (!sessionID || !sessionID[0])
         return;
 
-    m_client->requestAutomationSession(String::fromUTF8(sessionID));
+    m_client->requestAutomationSession(String::fromUTF8(sessionID), capabilities);
     updateClientCapabilities();
 }
 
