@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012 Apple Inc. All rights reserved.
+ * Copyright (C) 2012-2018 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -27,12 +27,9 @@
 
 namespace JSC {
 
-enum UnwindStart : uint8_t;
-
 class ExecState;
 class VM;
 
-void genericUnwind(VM*, ExecState*, UnwindStart);
 void genericUnwind(VM*, ExecState*);
 
 } // namespace JSC
