@@ -71,13 +71,6 @@ namespace JSC {
         {
         }
 
-        UnlinkedSourceCode(const UnlinkedSourceCode& other)
-            : m_provider(other.m_provider)
-            , m_startOffset(other.m_startOffset)
-            , m_endOffset(other.m_endOffset)
-        {
-        }
-
         bool isHashTableDeletedValue() const { return m_provider.isHashTableDeletedValue(); }
 
         const SourceProvider& provider() const
