@@ -28,48 +28,31 @@
 #define JSC_COMMON_IDENTIFIERS_EACH_PROPERTY_NAME(macro) \
     macro(Array) \
     macro(ArrayBuffer) \
-    macro(ArrayIterator) \
     macro(BYTES_PER_ELEMENT) \
     macro(BigInt) \
     macro(Boolean) \
     macro(Collator) \
-    macro(Credential) \
     macro(Date) \
     macro(DateTimeFormat) \
-    macro(DataTransferItem) \
-    macro(DataTransferItemList) \
     macro(Error) \
     macro(EvalError) \
     macro(Function) \
-    macro(GeneratorFunction) \
     macro(Infinity) \
     macro(Intl) \
-    macro(JSON) \
     macro(Loader) \
-    macro(Map)\
-    macro(MapIterator)\
-    macro(Math) \
+    macro(Map) \
     macro(NaN) \
     macro(Number) \
     macro(NumberFormat) \
     macro(Object) \
+    macro(PluralRules) \
     macro(Promise) \
-    macro(Proxy) \
-    macro(RangeError) \
-    macro(ReferenceError) \
     macro(Reflect) \
     macro(RegExp) \
-    macro(Set)\
-    macro(SetIterator)\
+    macro(Set) \
     macro(SharedArrayBuffer) \
     macro(String) \
     macro(Symbol) \
-    macro(SyntaxError) \
-    macro(TypeError) \
-    macro(URIError) \
-    macro(UTC) \
-    macro(WeakMap)\
-    macro(WeakSet)\
     macro(__defineGetter__) \
     macro(__defineSetter__) \
     macro(__lookupGetter__) \
@@ -79,11 +62,9 @@
     macro(anonymous) \
     macro(arguments) \
     macro(as) \
-    macro(assign) \
     macro(async) \
     macro(back) \
     macro(bind) \
-    macro(buffer) \
     macro(byteLength) \
     macro(byteOffset) \
     macro(bytecode) \
@@ -120,7 +101,6 @@
     macro(exec) \
     macro(executionCount) \
     macro(exitKind) \
-    macro(fetch) \
     macro(flags) \
     macro(forEach) \
     macro(formatMatcher) \
@@ -137,8 +117,8 @@
     macro(hash) \
     macro(header) \
     macro(hour) \
+    macro(hourCycle) \
     macro(hour12) \
-    macro(href) \
     macro(id) \
     macro(ignoreCase) \
     macro(ignorePunctuation) \
@@ -182,7 +162,6 @@
     macro(propertyIsEnumerable) \
     macro(prototype) \
     macro(raw) \
-    macro(reload) \
     macro(replace) \
     macro(resolve) \
     macro(second) \
@@ -283,6 +262,16 @@
     macro(toStringTag) \
     macro(unscopables)
 
+#define JSC_PARSER_PRIVATE_NAMES(macro) \
+    macro(generator) \
+    macro(generatorState) \
+    macro(generatorValue) \
+    macro(generatorResumeMode) \
+    macro(generatorFrame) \
+    macro(meta) \
+    macro(starDefault) \
+    macro(undefined) \
+
 namespace JSC {
     
     class BuiltinNames;
@@ -306,6 +295,10 @@ namespace JSC {
         std::unique_ptr<BuiltinNames> m_builtinNames;
 
     public:
+
+#define JSC_IDENTIFIER_DECLARE_PARSER_PRIVATE_NAME(name) const Identifier name##PrivateName;
+        JSC_PARSER_PRIVATE_NAMES(JSC_IDENTIFIER_DECLARE_PARSER_PRIVATE_NAME)
+#undef JSC_IDENTIFIER_DECLARE_PARSER_PRIVATE_NAME
         
 #define JSC_IDENTIFIER_DECLARE_KEYWORD_NAME_GLOBAL(name) const Identifier name##Keyword;
         JSC_COMMON_IDENTIFIERS_EACH_KEYWORD(JSC_IDENTIFIER_DECLARE_KEYWORD_NAME_GLOBAL)
@@ -319,8 +312,8 @@ namespace JSC {
         JSC_COMMON_PRIVATE_IDENTIFIERS_EACH_WELL_KNOWN_SYMBOL(JSC_IDENTIFIER_DECLARE_PRIVATE_WELL_KNOWN_SYMBOL_GLOBAL)
 #undef JSC_IDENTIFIER_DECLARE_PRIVATE_WELL_KNOWN_SYMBOL_GLOBAL
 
-        const Identifier* lookUpPrivateName(const Identifier&) const;
-        Identifier lookUpPublicName(const Identifier&) const;
+        SymbolImpl* lookUpPrivateName(const Identifier&) const;
+        Identifier getPublicName(VM&, SymbolImpl*) const;
 
         // Callers of this method should make sure that identifiers given to this method 
         // survive the lifetime of CommonIdentifiers and related VM.
