@@ -491,14 +491,13 @@ void ConfigFile::canonicalizePaths()
                     strncat(filenameBuffer, "/", 2); // Room for '/' plus NUL
 #if COMPILER(GCC)
 #if GCC_VERSION_AT_LEAST(8, 0, 0)
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wstringop-truncation"
+                IGNORE_WARNINGS_BEGIN("stringop-truncation")
 #endif
 #endif
                 strncat(filenameBuffer, m_filename, sizeof(filenameBuffer) - strlen(filenameBuffer) - 1);
 #if COMPILER(GCC)
 #if GCC_VERSION_AT_LEAST(8, 0, 0)
-#pragma GCC diagnostic pop
+                IGNORE_WARNINGS_END
 #endif
 #endif
                 strncpy(m_filename, filenameBuffer, s_maxPathLength);
