@@ -31,10 +31,10 @@
 
 namespace JSC {
     
-class Exception : public JSDestructibleObject {
+class Exception final : public JSDestructibleObject {
 public:
     typedef JSDestructibleObject Base;
-    static const unsigned StructureFlags = StructureIsImmortal | Base::StructureFlags;
+    static const unsigned StructureFlags = Base::StructureFlags | StructureIsImmortal;
 
     enum StackCaptureAction {
         CaptureStack,
