@@ -44,24 +44,38 @@ class GetterSetter final : public JSNonFinalObject {
     friend class JIT;
     typedef JSNonFinalObject Base;
 private:
-    GetterSetter(VM& vm, JSGlobalObject* globalObject)
+    GetterSetter(VM& vm, JSGlobalObject* globalObject, JSObject* getter, JSObject* setter)
         : Base(vm, globalObject->getterSetterStructure())
     {
-        m_getter.set(vm, this, globalObject->nullGetterFunction());
-        m_setter.set(vm, this, globalObject->nullSetterFunction());
+        WTF::storeStoreFence();
+        m_getter.set(vm, this, getter ? getter : globalObject->nullGetterFunction());
+        m_setter.set(vm, this, setter ? setter : globalObject->nullSetterFunction());
     }
 
 public:
 
     static const unsigned StructureFlags = Base::StructureFlags | OverridesGetOwnPropertySlot | StructureIsImmortal;
 
-    static GetterSetter* create(VM& vm, JSGlobalObject* globalObject)
+    static GetterSetter* create(VM& vm, JSGlobalObject* globalObject, JSObject* getter, JSObject* setter)
     {
-        GetterSetter* getterSetter = new (NotNull, allocateCell<GetterSetter>(vm.heap)) GetterSetter(vm, globalObject);
+        GetterSetter* getterSetter = new (NotNull, allocateCell<GetterSetter>(vm.heap)) GetterSetter(vm, globalObject, getter, setter);
         getterSetter->finishCreation(vm);
         return getterSetter;
     }
 
+    static GetterSetter* create(VM& vm, JSGlobalObject* globalObject, JSValue getter, JSValue setter)
+    {
+        ASSERT(getter.isUndefined() || getter.isObject());
+        ASSERT(setter.isUndefined() || setter.isObject());
+        JSObject* getterObject { nullptr };
+        JSObject* setterObject { nullptr };
+        if (getter.isObject())
+            getterObject = asObject(getter);
+        if (setter.isObject())
+            setterObject = asObject(setter);
+        return create(vm, globalObject, getterObject, setterObject);
+    }
+
     static void visitChildren(JSCell*, SlotVisitor&);
 
     JSObject* getter() const { return m_getter.get(); }
@@ -76,18 +90,6 @@ public:
     bool isGetterNull() const { return !!jsDynamicCast<NullGetterFunction*>(*m_getter.get()->vm(), m_getter.get()); }
     bool isSetterNull() const { return !!jsDynamicCast<NullSetterFunction*>(*m_setter.get()->vm(), m_setter.get()); }
 
-    // Set the getter. It's only valid to call this if you've never set the getter on this
-    // object.
-    void setGetter(VM& vm, JSGlobalObject* globalObject, JSObject* getter)
-    {
-        if (!getter)
-            getter = jsCast<JSObject*>(globalObject->nullGetterFunction());
-
-        RELEASE_ASSERT(isGetterNull());
-        WTF::storeStoreFence();
-        m_getter.set(vm, this, getter);
-    }
-
     JSObject* setter() const { return m_setter.get(); }
 
     JSObject* setterConcurrently() const
@@ -97,21 +99,6 @@ public:
         return result;
     }
 
-    // Set the setter. It's only valid to call this if you've never set the setter on this
-    // object.
-    void setSetter(VM& vm, JSGlobalObject* globalObject, JSObject* setter)
-    {
-        if (!setter)
-            setter = jsCast<JSObject*>(globalObject->nullSetterFunction());
-
-        RELEASE_ASSERT(isSetterNull());
-        WTF::storeStoreFence();
-        m_setter.set(vm, this, setter);
-    }
-
-    GetterSetter* withGetter(VM&, JSGlobalObject*, JSObject* getter);
-    GetterSetter* withSetter(VM&, JSGlobalObject*, JSObject* setter);
-
     static Structure* createStructure(VM& vm, JSGlobalObject* globalObject, JSValue prototype)
     {
         return Structure::create(vm, globalObject, prototype, TypeInfo(GetterSetterType), info());
@@ -131,6 +118,8 @@ public:
 
     static bool getOwnPropertySlot(JSObject*, ExecState*, PropertyName, PropertySlot&) { RELEASE_ASSERT_NOT_REACHED(); return false; }
     static bool put(JSCell*, ExecState*, PropertyName, JSValue, PutPropertySlot&) { RELEASE_ASSERT_NOT_REACHED(); return false; }
+    static bool putByIndex(JSCell*, ExecState*, unsigned, JSValue, bool) { RELEASE_ASSERT_NOT_REACHED(); return false; }
+    static bool setPrototype(JSObject*, ExecState*, JSValue, bool) { RELEASE_ASSERT_NOT_REACHED(); return false; }
     static bool defineOwnProperty(JSObject*, ExecState*, PropertyName, const PropertyDescriptor&, bool) { RELEASE_ASSERT_NOT_REACHED(); return false; }
     static bool deleteProperty(JSCell*, ExecState*, PropertyName) { RELEASE_ASSERT_NOT_REACHED(); return false; }
 
@@ -139,14 +128,6 @@ private:
     WriteBarrier<JSObject> m_setter;  
 };
 
-GetterSetter* asGetterSetter(JSValue);
-
-inline GetterSetter* asGetterSetter(JSValue value)
-{
-    ASSERT_WITH_SECURITY_IMPLICATION(value.asCell()->isGetterSetter());
-    return static_cast<GetterSetter*>(value.asCell());
-}
-
 JSValue callGetter(ExecState*, JSValue base, JSValue getterSetter);
 JS_EXPORT_PRIVATE bool callSetter(ExecState*, JSValue base, JSValue getterSetter, JSValue, ECMAMode);
 
