@@ -30,7 +30,7 @@
 namespace JSC {
 
 template<typename ViewClass>
-class JSGenericTypedArrayViewConstructor : public InternalFunction {
+class JSGenericTypedArrayViewConstructor final : public InternalFunction {
 public:
     typedef InternalFunction Base;
 
@@ -43,19 +43,10 @@ public:
         VM&, JSGlobalObject*, Structure*, JSObject* prototype, const String& name, FunctionExecutable* privateAllocator);
 
     // FIXME: We should fix the warnings for extern-template in JSObject template classes: https://bugs.webkit.org/show_bug.cgi?id=161979
-#if COMPILER(CLANG)
-#if __has_warning("-Wundefined-var-template")
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wundefined-var-template"
-#endif
-#endif
+    IGNORE_CLANG_WARNINGS_BEGIN("undefined-var-template")
     DECLARE_INFO;
-#if COMPILER(CLANG)
-#if __has_warning("-Wundefined-var-template")
-#pragma clang diagnostic pop
-#endif
-#endif
-    
+    IGNORE_CLANG_WARNINGS_END
+
     static Structure* createStructure(VM&, JSGlobalObject*, JSValue prototype);
 };
 
