@@ -89,9 +89,10 @@ ALWAYS_INLINE static JSMap* getMap(CallFrame* callFrame, JSValue thisValue)
         return nullptr;
     }
 
-    if (LIKELY(thisValue.asCell()->type() == JSMapType))
-        return jsCast<JSMap*>(thisValue);
-    throwTypeError(callFrame, scope, ASCIILiteral("Map operation called on non-Map object"));
+    auto* map = jsDynamicCast<JSMap*>(vm, thisValue.asCell());
+    if (LIKELY(map))
+        return map;
+    throwTypeError(callFrame, scope, "Map operation called on non-Map object"_s);
     return nullptr;
 }
 
