@@ -25,7 +25,7 @@
 
 #pragma once
 
-#include "RegExpObject.h"
+#include "RegExp.h"
 
 namespace JSC {
 
@@ -43,14 +43,6 @@ class JSString;
 // m_reifiedResult and m_reifiedInput hold the cached results.
 class RegExpCachedResult {
 public:
-    RegExpCachedResult(VM& vm, JSObject* owner, RegExp* emptyRegExp)
-        : m_result(0, 0)
-        , m_reified(false)
-    {
-        m_lastInput.set(vm, owner, jsEmptyString(&vm));
-        m_lastRegExp.set(vm, owner, emptyRegExp);
-    }
-
     ALWAYS_INLINE void record(VM& vm, JSObject* owner, RegExp* regExp, JSString* input, MatchResult result)
     {
         vm.heap.writeBarrier(owner);
@@ -71,16 +63,18 @@ public:
         return m_reified ? m_reifiedInput.get() : m_lastInput.get();
     }
 
-    void visitChildren(SlotVisitor&);
+    void visitAggregate(SlotVisitor&);
 
+    // m_lastRegExp would be nullptr when RegExpCachedResult is not reified.
+    // If we find m_lastRegExp is nullptr, it means this should hold the empty RegExp.
     static ptrdiff_t offsetOfLastRegExp() { return OBJECT_OFFSETOF(RegExpCachedResult, m_lastRegExp); }
     static ptrdiff_t offsetOfLastInput() { return OBJECT_OFFSETOF(RegExpCachedResult, m_lastInput); }
     static ptrdiff_t offsetOfResult() { return OBJECT_OFFSETOF(RegExpCachedResult, m_result); }
     static ptrdiff_t offsetOfReified() { return OBJECT_OFFSETOF(RegExpCachedResult, m_reified); }
 
 private:
-    MatchResult m_result;
-    bool m_reified;
+    MatchResult m_result { 0, 0 };
+    bool m_reified { false };
     WriteBarrier<JSString> m_lastInput;
     WriteBarrier<RegExp> m_lastRegExp;
     WriteBarrier<JSArray> m_reifiedResult;
