@@ -55,19 +55,19 @@ intptr_t StackFrame::sourceID() const
 {
     if (!m_codeBlock)
         return noSourceID;
-    return m_codeBlock->ownerScriptExecutable()->sourceID();
+    return m_codeBlock->ownerExecutable()->sourceID();
 }
 
 String StackFrame::sourceURL() const
 {
     if (m_isWasmFrame)
-        return ASCIILiteral("[wasm code]");
+        return "[wasm code]"_s;
 
     if (!m_codeBlock) {
-        return ASCIILiteral("[native code]");
+        return "[native code]"_s;
     }
 
-    String sourceURL = m_codeBlock->ownerScriptExecutable()->sourceURL();
+    String sourceURL = m_codeBlock->ownerExecutable()->sourceURL();
     if (!sourceURL.isNull())
         return sourceURL;
     return emptyString();
@@ -81,13 +81,13 @@ String StackFrame::functionName(VM& vm) const
     if (m_codeBlock) {
         switch (m_codeBlock->codeType()) {
         case EvalCode:
-            return ASCIILiteral("eval code");
+            return "eval code"_s;
         case ModuleCode:
-            return ASCIILiteral("module code");
+            return "module code"_s;
         case FunctionCode:
             break;
         case GlobalCode:
-            return ASCIILiteral("global code");
+            return "global code"_s;
         default:
             ASSERT_NOT_REACHED();
         }
@@ -113,9 +113,9 @@ void StackFrame::computeLineAndColumn(unsigned& line, unsigned& column) const
     int unusedEndOffset = 0;
     m_codeBlock->expressionRangeForBytecodeOffset(m_bytecodeOffset, divot, unusedStartOffset, unusedEndOffset, line, column);
 
-    ScriptExecutable* executable = m_codeBlock->ownerScriptExecutable();
-    if (executable->hasOverrideLineNumber())
-        line = executable->overrideLineNumber();
+    ScriptExecutable* executable = m_codeBlock->ownerExecutable();
+    if (Optional<int> overrideLineNumber = executable->overrideLineNumber(*m_codeBlock->vm()))
+        line = overrideLineNumber.value();
 }
 
 String StackFrame::toString(VM& vm) const
