@@ -15,15 +15,16 @@
 #include "libANGLE/renderer/null/CompilerNULL.h"
 #include "libANGLE/renderer/null/DisplayNULL.h"
 #include "libANGLE/renderer/null/FenceNVNULL.h"
-#include "libANGLE/renderer/null/FenceSyncNULL.h"
 #include "libANGLE/renderer/null/FramebufferNULL.h"
 #include "libANGLE/renderer/null/ImageNULL.h"
 #include "libANGLE/renderer/null/PathNULL.h"
 #include "libANGLE/renderer/null/ProgramNULL.h"
+#include "libANGLE/renderer/null/ProgramPipelineNULL.h"
 #include "libANGLE/renderer/null/QueryNULL.h"
 #include "libANGLE/renderer/null/RenderbufferNULL.h"
 #include "libANGLE/renderer/null/SamplerNULL.h"
 #include "libANGLE/renderer/null/ShaderNULL.h"
+#include "libANGLE/renderer/null/SyncNULL.h"
 #include "libANGLE/renderer/null/TextureNULL.h"
 #include "libANGLE/renderer/null/TransformFeedbackNULL.h"
 #include "libANGLE/renderer/null/VertexArrayNULL.h"
@@ -63,14 +64,34 @@ ContextNULL::ContextNULL(const gl::ContextState &state, AllocationTrackerNULL *a
 {
     ASSERT(mAllocationTracker != nullptr);
 
-    const gl::Version maxClientVersion(3, 1);
-    mCaps        = GenerateMinimumCaps(maxClientVersion);
-
-    mExtensions  = gl::Extensions();
+    mExtensions                       = gl::Extensions();
+    mExtensions.fence                 = true;
+    mExtensions.instancedArrays       = true;
+    mExtensions.pixelBufferObject     = true;
+    mExtensions.mapBuffer             = true;
+    mExtensions.mapBufferRange        = true;
     mExtensions.copyTexture           = true;
     mExtensions.copyCompressedTexture = true;
+    mExtensions.textureRectangle      = true;
+    mExtensions.textureUsage           = true;
+    mExtensions.vertexArrayObject      = true;
+    mExtensions.debugMarker            = true;
+    mExtensions.translatedShaderSource = true;
 
-    mTextureCaps = GenerateMinimumTextureCapsMap(maxClientVersion, mExtensions);
+    mExtensions.rgb8rgba8 = true;
+    mExtensions.textureCompressionDXT1     = true;
+    mExtensions.textureCompressionDXT3     = true;
+    mExtensions.textureCompressionDXT5     = true;
+    mExtensions.textureCompressionS3TCsRGB = true;
+    mExtensions.textureCompressionASTCHDR  = true;
+    mExtensions.textureCompressionASTCLDR  = true;
+    mExtensions.compressedETC1RGB8Texture  = true;
+    mExtensions.lossyETCDecode             = true;
+
+    const gl::Version maxClientVersion(3, 1);
+    mCaps = GenerateMinimumCaps(maxClientVersion, mExtensions);
+
+    InitMinimumTextureCapsMap(maxClientVersion, mExtensions, &mTextureCaps);
 }
 
 ContextNULL::~ContextNULL()
@@ -82,22 +103,26 @@ gl::Error ContextNULL::initialize()
     return gl::NoError();
 }
 
-gl::Error ContextNULL::flush()
+gl::Error ContextNULL::flush(const gl::Context *context)
 {
     return gl::NoError();
 }
 
-gl::Error ContextNULL::finish()
+gl::Error ContextNULL::finish(const gl::Context *context)
 {
     return gl::NoError();
 }
 
-gl::Error ContextNULL::drawArrays(GLenum mode, GLint first, GLsizei count)
+gl::Error ContextNULL::drawArrays(const gl::Context *context,
+                                  GLenum mode,
+                                  GLint first,
+                                  GLsizei count)
 {
     return gl::NoError();
 }
 
-gl::Error ContextNULL::drawArraysInstanced(GLenum mode,
+gl::Error ContextNULL::drawArraysInstanced(const gl::Context *context,
+                                           GLenum mode,
                                            GLint first,
                                            GLsizei count,
                                            GLsizei instanceCount)
@@ -105,42 +130,47 @@ gl::Error ContextNULL::drawArraysInstanced(GLenum mode,
     return gl::NoError();
 }
 
-gl::Error ContextNULL::drawElements(GLenum mode,
+gl::Error ContextNULL::drawElements(const gl::Context *context,
+                                    GLenum mode,
                                     GLsizei count,
                                     GLenum type,
-                                    const GLvoid *indices,
-                                    const gl::IndexRange &indexRange)
+                                    const void *indices)
 {
     return gl::NoError();
 }
 
-gl::Error ContextNULL::drawElementsInstanced(GLenum mode,
+gl::Error ContextNULL::drawElementsInstanced(const gl::Context *context,
+                                             GLenum mode,
                                              GLsizei count,
                                              GLenum type,
-                                             const GLvoid *indices,
-                                             GLsizei instances,
-                                             const gl::IndexRange &indexRange)
+                                             const void *indices,
+                                             GLsizei instances)
 {
     return gl::NoError();
 }
 
-gl::Error ContextNULL::drawRangeElements(GLenum mode,
+gl::Error ContextNULL::drawRangeElements(const gl::Context *context,
+                                         GLenum mode,
                                          GLuint start,
                                          GLuint end,
                                          GLsizei count,
                                          GLenum type,
-                                         const GLvoid *indices,
-                                         const gl::IndexRange &indexRange)
+                                         const void *indices)
 {
     return gl::NoError();
 }
 
-gl::Error ContextNULL::drawArraysIndirect(GLenum mode, const GLvoid *indirect)
+gl::Error ContextNULL::drawArraysIndirect(const gl::Context *context,
+                                          GLenum mode,
+                                          const void *indirect)
 {
     return gl::NoError();
 }
 
-gl::Error ContextNULL::drawElementsIndirect(GLenum mode, GLenum type, const GLvoid *indirect)
+gl::Error ContextNULL::drawElementsIndirect(const gl::Context *context,
+                                            GLenum mode,
+                                            GLenum type,
+                                            const void *indirect)
 {
     return gl::NoError();
 }
@@ -250,7 +280,15 @@ void ContextNULL::popGroupMarker()
 {
 }
 
-void ContextNULL::syncState(const gl::State::DirtyBits &dirtyBits)
+void ContextNULL::pushDebugGroup(GLenum source, GLuint id, GLsizei length, const char *message)
+{
+}
+
+void ContextNULL::popDebugGroup()
+{
+}
+
+void ContextNULL::syncState(const gl::Context *context, const gl::State::DirtyBits &dirtyBits)
 {
 }
 
@@ -264,7 +302,7 @@ GLint64 ContextNULL::getTimestamp()
     return 0;
 }
 
-void ContextNULL::onMakeCurrent(const gl::ContextState &data)
+void ContextNULL::onMakeCurrent(const gl::Context *context)
 {
 }
 
@@ -338,9 +376,9 @@ FenceNVImpl *ContextNULL::createFenceNV()
     return new FenceNVNULL();
 }
 
-FenceSyncImpl *ContextNULL::createFenceSync()
+SyncImpl *ContextNULL::createSync()
 {
-    return new FenceSyncNULL();
+    return new SyncNULL();
 }
 
 TransformFeedbackImpl *ContextNULL::createTransformFeedback(const gl::TransformFeedbackState &state)
@@ -348,9 +386,14 @@ TransformFeedbackImpl *ContextNULL::createTransformFeedback(const gl::TransformF
     return new TransformFeedbackNULL(state);
 }
 
-SamplerImpl *ContextNULL::createSampler()
+SamplerImpl *ContextNULL::createSampler(const gl::SamplerState &state)
 {
-    return new SamplerNULL();
+    return new SamplerNULL(state);
+}
+
+ProgramPipelineImpl *ContextNULL::createProgramPipeline(const gl::ProgramPipelineState &state)
+{
+    return new ProgramPipelineNULL(state);
 }
 
 std::vector<PathImpl *> ContextNULL::createPaths(GLsizei range)
@@ -363,7 +406,10 @@ std::vector<PathImpl *> ContextNULL::createPaths(GLsizei range)
     return result;
 }
 
-gl::Error ContextNULL::dispatchCompute(GLuint numGroupsX, GLuint numGroupsY, GLuint numGroupsZ)
+gl::Error ContextNULL::dispatchCompute(const gl::Context *context,
+                                       GLuint numGroupsX,
+                                       GLuint numGroupsY,
+                                       GLuint numGroupsZ)
 {
     return gl::NoError();
 }
