@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016 Apple Inc. All rights reserved.
+ * Copyright (C) 2016-2018 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,6 +26,10 @@
 #pragma once
 
 #include <Block.h>
+#include <utility>
+#include <wtf/Assertions.h>
+#include <wtf/StdLibExtras.h>
+
 #if CPU(ARM64E)
 #include <WebKitAdditions/BlockQualifiers.h>
 #else
@@ -33,8 +37,6 @@
 #define WTF_DISPOSE_FUNCTION_POINTER_QUALIFIER
 #define WTF_INVOKE_FUNCTION_POINTER_QUALIFIER
 #endif
-#include <utility>
-#include <wtf/Assertions.h>
 
 namespace WTF {
 
@@ -97,10 +99,13 @@ public:
 
         new (&block->f) F { std::move(function) };
 
+#if __has_feature(objc_arc)
+        return BlockPtr { (__bridge_transfer BlockType)block };
+#else
         BlockPtr blockPtr;
         blockPtr.m_block = reinterpret_cast<BlockType>(block);
-
         return blockPtr;
+#endif
     }
 
     BlockPtr()
@@ -109,12 +114,20 @@ public:
     }
 
     BlockPtr(BlockType block)
+#if __has_feature(objc_arc)
+        : m_block(WTFMove(block))
+#else
         : m_block(Block_copy(block))
+#endif
     {
     }
 
     BlockPtr(const BlockPtr& other)
+#if __has_feature(objc_arc)
+        : m_block(other.m_block)
+#else
         : m_block(Block_copy(other.m_block))
+#endif
     {
     }
     
@@ -125,16 +138,22 @@ public:
     
     ~BlockPtr()
     {
+#if !__has_feature(objc_arc)
         Block_release(m_block);
+#endif
     }
 
     BlockPtr& operator=(const BlockPtr& other)
     {
+#if __has_feature(objc_arc)
+        m_block = other.m_block;
+#else
         if (this != &other) {
             Block_release(m_block);
             m_block = Block_copy(other.m_block);
         }
-        
+#endif
+
         return *this;
     }
 
@@ -142,7 +161,9 @@ public:
     {
         ASSERT(this != &other);
 
+#if !__has_feature(objc_arc)
         Block_release(m_block);
+#endif
         m_block = std::exchange(other.m_block, nullptr);
 
         return *this;
@@ -170,8 +191,25 @@ inline BlockPtr<R (Args...)> makeBlockPtr(R (^block)(Args...))
     return BlockPtr<R (Args...)>(block);
 }
 
+template<typename F, typename Class, typename R, typename... Args>
+inline auto makeBlockPtr(F&& function, R (Class::*)(Args...) const)
+{
+    return BlockPtr<R (Args...)>::fromCallable(std::forward<F>(function));
+}
+
+template<typename F, typename Class, typename R, typename... Args>
+inline auto makeBlockPtr(F&& function, R (Class::*)(Args...))
+{
+    return BlockPtr<R (Args...)>::fromCallable(std::forward<F>(function));
+}
+
+template<typename F>
+inline auto makeBlockPtr(F&& function)
+{
+    return makeBlockPtr(std::forward<F>(function), &F::operator());
+}
+
 }
 
 using WTF::BlockPtr;
 using WTF::makeBlockPtr;
-
