@@ -37,10 +37,18 @@
 #define DYLD_IOS_VERSION_11_3 0x000B0300
 #endif
 
+#ifndef DYLD_IOS_VERSION_12_0
+#define DYLD_IOS_VERSION_12_0 0x000C0000
+#endif
+
 #ifndef DYLD_MACOSX_VERSION_10_13
 #define DYLD_MACOSX_VERSION_10_13 0x000A0D00
 #endif
 
+#ifndef DYLD_MACOSX_VERSION_10_14
+#define DYLD_MACOSX_VERSION_10_14 0x000A0E00
+#endif
+
 #else
 
 #define DYLD_IOS_VERSION_3_0 0x00030000
@@ -52,10 +60,12 @@
 #define DYLD_IOS_VERSION_10_0 0x000A0000
 #define DYLD_IOS_VERSION_11_0 0x000B0000
 #define DYLD_IOS_VERSION_11_3 0x000B0300
+#define DYLD_IOS_VERSION_12_0 0x000C0000
 
 #define DYLD_MACOSX_VERSION_10_11 0x000A0B00
 #define DYLD_MACOSX_VERSION_10_12 0x000A0C00
 #define DYLD_MACOSX_VERSION_10_13 0x000A0D00
+#define DYLD_MACOSX_VERSION_10_14 0x000A0E00
 
 #endif
 
