@@ -278,7 +278,7 @@ String tryMakeStringFromAdapters(StringTypeAdapter adapter, StringTypeAdapters .
 
         makeStringAccumulator(buffer, adapter, adapters...);
 
-        return WTFMove(resultImpl);
+        return resultImpl;
     }
 
     UChar* buffer;
@@ -288,7 +288,7 @@ String tryMakeStringFromAdapters(StringTypeAdapter adapter, StringTypeAdapters .
 
     makeStringAccumulator(buffer, adapter, adapters...);
 
-    return WTFMove(resultImpl);
+    return resultImpl;
 }
 
 template<typename... StringTypes>
