@@ -90,11 +90,36 @@ public:
     WTF_EXPORT_PRIVATE void endGroup();
     WTF_EXPORT_PRIVATE void nextLine(); // Output newline and indent.
 
+    int indent() const { return m_indent; }
     void increaseIndent(int amount = 1) { m_indent += amount; }
     void decreaseIndent(int amount = 1) { m_indent -= amount; ASSERT(m_indent >= 0); }
 
     WTF_EXPORT_PRIVATE void writeIndent();
 
+    // Stream manipulators.
+    TextStream& operator<<(TextStream& (*func)(TextStream&))
+    {
+        return (*func)(*this);
+    }
+
+    class IndentScope {
+    public:
+        IndentScope(TextStream& ts, int amount = 1)
+            : m_stream(ts)
+            , m_amount(amount)
+        {
+            m_stream.increaseIndent(m_amount);
+        }
+        ~IndentScope()
+        {
+            m_stream.decreaseIndent(m_amount);
+        }
+
+    private:
+        TextStream& m_stream;
+        int m_amount;
+    };
+
     class GroupScope {
     public:
         GroupScope(TextStream& ts)
@@ -118,6 +143,12 @@ private:
     bool m_multiLineMode { true };
 };
 
+inline TextStream& indent(TextStream& ts)
+{
+    ts.writeIndent();
+    return ts;
+}
+
 template<typename Item>
 TextStream& operator<<(TextStream& ts, const Vector<Item>& vector)
 {
@@ -133,8 +164,10 @@ TextStream& operator<<(TextStream& ts, const Vector<Item>& vector)
     return ts << "]";
 }
 
+// Deprecated. Use TextStream::writeIndent() instead.
 WTF_EXPORT_PRIVATE void writeIndent(TextStream&, int indent);
 
 } // namespace WTF
 
 using WTF::TextStream;
+using WTF::indent;
