@@ -26,6 +26,7 @@
 
 #include "NavigatorGeolocation.h"
 
+#include "DOMWindow.h"
 #include "Document.h"
 #include "Frame.h"
 #include "Geolocation.h"
@@ -33,8 +34,8 @@
 
 namespace WebCore {
 
-NavigatorGeolocation::NavigatorGeolocation(Frame* frame)
-    : DOMWindowProperty(frame)
+NavigatorGeolocation::NavigatorGeolocation(DOMWindow* window)
+    : DOMWindowProperty(window)
 {
 }
 
@@ -49,20 +50,20 @@ NavigatorGeolocation* NavigatorGeolocation::from(Navigator* navigator)
 {
     NavigatorGeolocation* supplement = static_cast<NavigatorGeolocation*>(Supplement<Navigator>::from(navigator, supplementName()));
     if (!supplement) {
-        auto newSupplement = std::make_unique<NavigatorGeolocation>(navigator->frame());
+        auto newSupplement = std::make_unique<NavigatorGeolocation>(navigator->window());
         supplement = newSupplement.get();
         provideTo(navigator, supplementName(), WTFMove(newSupplement));
     }
     return supplement;
 }
 
-#if PLATFORM(IOS)
+#if PLATFORM(IOS_FAMILY)
 void NavigatorGeolocation::resetAllGeolocationPermission()
 {
     if (m_geolocation)
         m_geolocation->resetAllGeolocationPermission();
 }
-#endif // PLATFORM(IOS)
+#endif // PLATFORM(IOS_FAMILY)
 
 Geolocation* NavigatorGeolocation::geolocation(Navigator& navigator)
 {
@@ -71,8 +72,8 @@ Geolocation* NavigatorGeolocation::geolocation(Navigator& navigator)
 
 Geolocation* NavigatorGeolocation::geolocation() const
 {
-    if (!m_geolocation && frame())
-        m_geolocation = Geolocation::create(frame()->document());
+    if (!m_geolocation)
+        m_geolocation = Geolocation::create(window() ? window()->document() : nullptr);
     return m_geolocation.get();
 }
 
