@@ -115,6 +115,9 @@ void IDBOpenDBRequest::fireErrorAfterVersionChangeCompletion()
 void IDBOpenDBRequest::cancelForStop()
 {
     connectionProxy().openDBRequestCancelled({ connectionProxy(), *this });
+
+    if (m_transaction && m_transaction->isVersionChange())
+        m_transaction->removeRequest(*this);
 }
 
 void IDBOpenDBRequest::dispatchEvent(Event& event)
