@@ -27,11 +27,54 @@
 
 #if USE(LIBWEBRTC)
 
+#include "EventNames.h"
 #include "RTCDataChannel.h"
+#include "RTCDataChannelEvent.h"
 #include <wtf/MainThread.h>
 
 namespace WebCore {
 
+webrtc::DataChannelInit LibWebRTCDataChannelHandler::fromRTCDataChannelInit(const RTCDataChannelInit& options)
+{
+    webrtc::DataChannelInit init;
+    if (options.ordered)
+        init.ordered = *options.ordered;
+    if (options.maxPacketLifeTime)
+        init.maxRetransmitTime = *options.maxPacketLifeTime;
+    if (options.maxRetransmits)
+        init.maxRetransmits = *options.maxRetransmits;
+    init.protocol = options.protocol.utf8().data();
+    if (options.negotiated)
+        init.negotiated = *options.negotiated;
+    if (options.id)
+        init.id = *options.id;
+    return init;
+}
+
+static inline String fromStdString(const std::string& value)
+{
+    return String::fromUTF8(value.data(), value.length());
+}
+
+Ref<RTCDataChannelEvent> LibWebRTCDataChannelHandler::channelEvent(ScriptExecutionContext& context, rtc::scoped_refptr<webrtc::DataChannelInterface>&& dataChannel)
+{
+    auto protocol = dataChannel->protocol();
+    auto label = dataChannel->label();
+
+    RTCDataChannelInit init;
+    init.ordered = dataChannel->ordered();
+    init.maxPacketLifeTime = dataChannel->maxRetransmitTime();
+    init.maxRetransmits = dataChannel->maxRetransmits();
+    init.protocol = fromStdString(protocol);
+    init.negotiated = dataChannel->negotiated();
+    init.id = dataChannel->id();
+
+    auto handler =  std::make_unique<LibWebRTCDataChannelHandler>(WTFMove(dataChannel));
+    auto channel = RTCDataChannel::create(context, WTFMove(handler), fromStdString(label), WTFMove(init));
+
+    return RTCDataChannelEvent::create(eventNames().datachannelEvent, Event::CanBubble::No, Event::IsCancelable::No, WTFMove(channel));
+}
+
 LibWebRTCDataChannelHandler::~LibWebRTCDataChannelHandler()
 {
     if (m_client)
@@ -43,11 +86,13 @@ void LibWebRTCDataChannelHandler::setClient(RTCDataChannelHandlerClient& client)
     ASSERT(!m_client);
     m_client = &client;
     m_channel->RegisterObserver(this);
+    checkState();
 }
 
 bool LibWebRTCDataChannelHandler::sendStringData(const String& text)
 {
-    return m_channel->Send({rtc::CopyOnWriteBuffer(text.utf8().data(), text.length()), false});
+    auto utf8Text = text.utf8();
+    return m_channel->Send({ rtc::CopyOnWriteBuffer(utf8Text.data(), utf8Text.length()), false });
 }
 
 bool LibWebRTCDataChannelHandler::sendRawData(const char* data, size_t length)
@@ -68,7 +113,11 @@ void LibWebRTCDataChannelHandler::OnStateChange()
 {
     if (!m_client)
         return;
+    checkState();
+}
 
+void LibWebRTCDataChannelHandler::checkState()
+{
     RTCDataChannelState state;
     switch (m_channel->state()) {
     case webrtc::DataChannelInterface::kConnecting:
@@ -96,12 +145,11 @@ void LibWebRTCDataChannelHandler::OnMessage(const webrtc::DataBuffer& buffer)
 
     std::unique_ptr<webrtc::DataBuffer> protectedBuffer(new webrtc::DataBuffer(buffer));
     callOnMainThread([protectedClient = makeRef(*m_client), buffer = WTFMove(protectedBuffer)] {
-        // FIXME: Ensure this is correct by adding some tests with non-ASCII characters.
-        const char* data = reinterpret_cast<const char*>(buffer->data.data());
+        const char* data = reinterpret_cast<const char*>(buffer->data.data<char>());
         if (buffer->binary)
             protectedClient->didReceiveRawData(data, buffer->size());
         else
-            protectedClient->didReceiveStringData(String(data, buffer->size()));
+            protectedClient->didReceiveStringData(String::fromUTF8(data, buffer->size()));
     });
 }
 
