@@ -29,23 +29,14 @@
 #if ENABLE(SPEECH_SYNTHESIS)
 
 namespace WebCore {
-    
-Ref<SpeechSynthesisEvent> SpeechSynthesisEvent::create()
-{
-    return adoptRef(*new SpeechSynthesisEvent());
-}
 
-Ref<SpeechSynthesisEvent> SpeechSynthesisEvent::create(const AtomicString& type, unsigned long charIndex, float elapsedTime, const String& name)
+Ref<SpeechSynthesisEvent> SpeechSynthesisEvent::create(const AtomicString& type, unsigned charIndex, float elapsedTime, const String& name)
 {
     return adoptRef(*new SpeechSynthesisEvent(type, charIndex, elapsedTime, name));
 }
 
-SpeechSynthesisEvent::SpeechSynthesisEvent()
-{
-}
-
-SpeechSynthesisEvent::SpeechSynthesisEvent(const AtomicString& type, unsigned long charIndex, float elapsedTime, const String& name)
-    : Event(type, false, false)
+SpeechSynthesisEvent::SpeechSynthesisEvent(const AtomicString& type, unsigned charIndex, float elapsedTime, const String& name)
+    : Event(type, CanBubble::No, IsCancelable::No)
     , m_charIndex(charIndex)
     , m_elapsedTime(elapsedTime)
     , m_name(name)
