@@ -37,7 +37,7 @@ namespace WebCore {
 
 class AudioContext;
     
-class MediaElementAudioSourceNode : public AudioNode, public AudioSourceProviderClient {
+class MediaElementAudioSourceNode final : public AudioNode, public AudioSourceProviderClient {
 public:
     static Ref<MediaElementAudioSourceNode> create(AudioContext&, HTMLMediaElement&);
 
