@@ -30,27 +30,15 @@
 #endif
 
 #include "PlatformExportMacros.h"
+#include <JavaScriptCore/JSExportMacros.h>
 #include <pal/ExportMacros.h>
-#include <runtime/JSExportMacros.h>
-
-#ifdef __APPLE__
-#define HAVE_FUNC_USLEEP 1
-#endif /* __APPLE__ */
-
-#if OS(WINDOWS)
-
-// CURL needs winsock, so don't prevent inclusion of it
-#if !USE(CURL)
-#ifndef _WINSOCKAPI_
-#define _WINSOCKAPI_ // Prevent inclusion of winsock.h in windows.h
-#endif
-#endif
-
-#endif /* OS(WINDOWS) */
 
 // Using CMake with Unix makefiles does not use prefix headers.
 #if PLATFORM(MAC) && defined(BUILDING_WITH_CMAKE)
 #include "WebCorePrefix.h"
+#ifndef JSC_API_AVAILABLE
+#define JSC_API_AVAILABLE(...)
+#endif
 #endif
 
 #ifdef __cplusplus
@@ -67,33 +55,6 @@
 
 #include <wtf/DisallowCType.h>
 
-#if COMPILER(MSVC)
-#define SKIP_STATIC_CONSTRUCTORS_ON_MSVC 1
-#else
-#define SKIP_STATIC_CONSTRUCTORS_ON_GCC 1
-#endif
-
-#if PLATFORM(WIN)
-#if PLATFORM(WIN_CAIRO)
-#undef USE_CG
-#define USE_CURL 1
-#ifndef _WINSOCKAPI_
-#define _WINSOCKAPI_ // Prevent inclusion of winsock.h in windows.h
-#endif
-#elif USE(DIRECT2D)
-#undef USE_CA
-#undef USE_CG
-#elif !USE(WINGDI)
-#define USE_CG 1
-#undef USE_CAIRO
-#undef USE_CURL
-#endif
-#endif
-
-#if PLATFORM(MAC) || PLATFORM(WPE)
-#define USE_NEW_THEME 1
-#endif
-
 #if USE(CG)
 #ifndef CGFLOAT_DEFINED
 #if (defined(__LP64__) && __LP64__) || (defined(__x86_64__) && __x86_64__) || defined(_M_X64) || defined(__amd64__)
