@@ -35,18 +35,20 @@
 #include "MathMLNames.h"
 #include "QualifiedName.h"
 #include "ShadowRoot.h"
+#include "TypedElementDescendantIterator.h"
 #include <JavaScriptCore/JSCJSValueInlines.h>
 #include <wtf/text/AtomicString.h>
 
 namespace WebCore {
 
-Ref<CustomElementRegistry> CustomElementRegistry::create(DOMWindow& window)
+Ref<CustomElementRegistry> CustomElementRegistry::create(DOMWindow& window, ScriptExecutionContext* scriptExecutionContext)
 {
-    return adoptRef(*new CustomElementRegistry(window));
+    return adoptRef(*new CustomElementRegistry(window, scriptExecutionContext));
 }
 
-CustomElementRegistry::CustomElementRegistry(DOMWindow& window)
-    : m_window(window)
+CustomElementRegistry::CustomElementRegistry(DOMWindow& window, ScriptExecutionContext* scriptExecutionContext)
+    : ContextDestructionObserver(scriptExecutionContext)
+    , m_window(window)
 {
 }
 
@@ -113,4 +115,25 @@ JSC::JSValue CustomElementRegistry::get(const AtomicString& name)
     return JSC::jsUndefined();
 }
 
+static void upgradeElementsInShadowIncludingDescendants(ContainerNode& root)
+{
+    for (auto& element : descendantsOfType<Element>(root)) {
+        if (element.isCustomElementUpgradeCandidate())
+            CustomElementReactionQueue::enqueueElementUpgradeIfDefined(element);
+        if (auto* shadowRoot = element.shadowRoot())
+            upgradeElementsInShadowIncludingDescendants(*shadowRoot);
+    }
+}
+
+void CustomElementRegistry::upgrade(Node& root)
+{
+    if (!is<ContainerNode>(root))
+        return;
+
+    if (is<Element>(root) && downcast<Element>(root).isCustomElementUpgradeCandidate())
+        CustomElementReactionQueue::enqueueElementUpgradeIfDefined(downcast<Element>(root));
+
+    upgradeElementsInShadowIncludingDescendants(downcast<ContainerNode>(root));
+}
+
 }
