@@ -25,7 +25,7 @@
 
 #pragma once
 
-#include "JSDOMPromiseDeferred.h"
+#include "ContextDestructionObserver.h"
 #include "QualifiedName.h"
 #include <wtf/HashMap.h>
 #include <wtf/text/AtomicString.h>
@@ -42,13 +42,15 @@ namespace WebCore {
 
 class CustomElementRegistry;
 class DOMWindow;
+class DeferredPromise;
 class Element;
 class JSCustomElementInterface;
+class Node;
 class QualifiedName;
 
-class CustomElementRegistry : public RefCounted<CustomElementRegistry> {
+class CustomElementRegistry : public RefCounted<CustomElementRegistry>, public ContextDestructionObserver {
 public:
-    static Ref<CustomElementRegistry> create(DOMWindow&);
+    static Ref<CustomElementRegistry> create(DOMWindow&, ScriptExecutionContext*);
     ~CustomElementRegistry();
 
     void addElementDefinition(Ref<JSCustomElementInterface>&&);
@@ -62,11 +64,12 @@ public:
     bool containsConstructor(const JSC::JSObject*) const;
 
     JSC::JSValue get(const AtomicString&);
+    void upgrade(Node& root);
 
     HashMap<AtomicString, Ref<DeferredPromise>>& promiseMap() { return m_promiseMap; }
 
 private:
-    CustomElementRegistry(DOMWindow&);
+    CustomElementRegistry(DOMWindow&, ScriptExecutionContext*);
 
     DOMWindow& m_window;
     HashMap<AtomicString, Ref<JSCustomElementInterface>> m_nameMap;
