@@ -75,26 +75,21 @@ static void callDefaultEventHandlersInBubblingOrder(Event& event, const EventPat
 
 static void dispatchEventInDOM(Event& event, const EventPath& path)
 {
-    // Trigger capturing event handlers, starting at the top and working our way down.
-    event.setEventPhase(Event::CAPTURING_PHASE);
-
-    for (size_t i = path.size() - 1; i > 0; --i) {
-        const EventContext& eventContext = path.contextAt(i);
+    // Invoke capturing event listeners in the reverse order.
+    for (size_t i = path.size(); i > 0; --i) {
+        const EventContext& eventContext = path.contextAt(i - 1);
         if (eventContext.currentTarget() == eventContext.target())
-            continue;
-        eventContext.handleLocalEvents(event);
+            event.setEventPhase(Event::AT_TARGET);
+        else
+            event.setEventPhase(Event::CAPTURING_PHASE);
+        eventContext.handleLocalEvents(event, EventTarget::EventInvokePhase::Capturing);
         if (event.propagationStopped())
             return;
     }
 
-    event.setEventPhase(Event::AT_TARGET);
-    path.contextAt(0).handleLocalEvents(event);
-    if (event.propagationStopped())
-        return;
-
-    // Trigger bubbling event handlers, starting at the bottom and working our way up.
+    // Invoke bubbling event listeners.
     size_t size = path.size();
-    for (size_t i = 1; i < size; ++i) {
+    for (size_t i = 0; i < size; ++i) {
         const EventContext& eventContext = path.contextAt(i);
         if (eventContext.currentTarget() == eventContext.target())
             event.setEventPhase(Event::AT_TARGET);
@@ -102,7 +97,7 @@ static void dispatchEventInDOM(Event& event, const EventPath& path)
             event.setEventPhase(Event::BUBBLING_PHASE);
         else
             continue;
-        eventContext.handleLocalEvents(event);
+        eventContext.handleLocalEvents(event, EventTarget::EventInvokePhase::Bubbling);
         if (event.propagationStopped())
             return;
     }
