@@ -34,21 +34,17 @@
 #include "MessagePortChannelProvider.h"
 #include "MessageWithMessagePorts.h"
 #include "WorkerGlobalScope.h"
+#include "WorkerThread.h"
 
 namespace WebCore {
 
+static Lock allMessagePortsLock;
 static HashMap<MessagePortIdentifier, MessagePort*>& allMessagePorts()
 {
     static NeverDestroyed<HashMap<MessagePortIdentifier, MessagePort*>> map;
     return map;
 }
 
-static Lock& allMessagePortsLock()
-{
-    static NeverDestroyed<Lock> lock;
-    return lock;
-}
-
 void MessagePort::ref() const
 {
     ++m_refCount;
@@ -60,26 +56,29 @@ void MessagePort::deref() const
     // This allows isExistingMessagePortLocallyReachable and notifyMessageAvailable to easily query the map and manipulate MessagePort instances.
 
     if (!--m_refCount) {
-        Locker<Lock> locker(allMessagePortsLock());
+        Locker<Lock> locker(allMessagePortsLock);
 
         if (m_refCount)
             return;
 
-        allMessagePorts().remove(m_identifier);
+        auto iterator = allMessagePorts().find(m_identifier);
+        if (iterator != allMessagePorts().end() && iterator->value == this)
+            allMessagePorts().remove(iterator);
+
         delete this;
     }
 }
 
 bool MessagePort::isExistingMessagePortLocallyReachable(const MessagePortIdentifier& identifier)
 {
-    Locker<Lock> locker(allMessagePortsLock());
+    Locker<Lock> locker(allMessagePortsLock);
     auto* port = allMessagePorts().get(identifier);
     return port && port->isLocallyReachable();
 }
 
 void MessagePort::notifyMessageAvailable(const MessagePortIdentifier& identifier)
 {
-    Locker<Lock> locker(allMessagePortsLock());
+    Locker<Lock> locker(allMessagePortsLock);
     if (auto* port = allMessagePorts().get(identifier))
         port->messageAvailable();
 
@@ -97,7 +96,7 @@ MessagePort::MessagePort(ScriptExecutionContext& scriptExecutionContext, const M
 {
     LOG(MessagePorts, "Created MessagePort %s (%p) in process %" PRIu64, m_identifier.logString().utf8().data(), this, Process::identifier().toUInt64());
 
-    Locker<Lock> locker(allMessagePortsLock());
+    Locker<Lock> locker(allMessagePortsLock);
     allMessagePorts().set(m_identifier, this);
 
     m_scriptExecutionContext->createdMessagePort(*this);
@@ -110,7 +109,7 @@ MessagePort::~MessagePort()
 {
     LOG(MessagePorts, "Destroyed MessagePort %s (%p) in process %" PRIu64, m_identifier.logString().utf8().data(), this, Process::identifier().toUInt64());
 
-    ASSERT(allMessagePortsLock().isLocked());
+    ASSERT(allMessagePortsLock.isLocked());
 
     if (m_entangled)
         close();
@@ -217,22 +216,20 @@ void MessagePort::start()
 void MessagePort::close()
 {
     m_mightBeEligibleForGC = true;
+
     if (m_closed)
         return;
+    m_closed = true;
 
     MessagePortChannelProvider::singleton().messagePortClosed(m_identifier);
-
-    m_closed = true;
+    removeAllEventListeners();
 }
 
 void MessagePort::contextDestroyed()
 {
     ASSERT(m_scriptExecutionContext);
 
-    m_mightBeEligibleForGC = true;
-    if (!m_closed)
-        close();
-
+    close();
     m_scriptExecutionContext = nullptr;
 }
 
@@ -245,8 +242,16 @@ void MessagePort::dispatchMessages()
     if (!isEntangled())
         return;
 
-    auto messagesTakenHandler = [this, protectedThis = makeRef(*this)](Vector<MessageWithMessagePorts>&& messages, Function<void()>&& completionCallback) mutable {
-        auto innerHandler = [this, otherProtectedThis = WTFMove(protectedThis)](Vector<MessageWithMessagePorts>&& messages) {
+    RefPtr<WorkerThread> workerThread;
+    if (is<WorkerGlobalScope>(*m_scriptExecutionContext))
+        workerThread = &downcast<WorkerGlobalScope>(*m_scriptExecutionContext).thread();
+
+    auto messagesTakenHandler = [this, weakThis = makeWeakPtr(this), workerThread = WTFMove(workerThread)](Vector<MessageWithMessagePorts>&& messages, Function<void()>&& completionCallback) mutable {
+        ASSERT(isMainThread());
+        auto innerHandler = [this, weakThis = WTFMove(weakThis)](auto&& messages) {
+            if (!weakThis)
+                return;
+
             LOG(MessagePorts, "MessagePort %s (%p) dispatching %zu messages", m_identifier.logString().utf8().data(), this, messages.size());
 
             if (!m_scriptExecutionContext)
@@ -263,30 +268,40 @@ void MessagePort::dispatchMessages()
                 if (contextIsWorker && downcast<WorkerGlobalScope>(*m_scriptExecutionContext).isClosing())
                     return;
                 auto ports = MessagePort::entanglePorts(*m_scriptExecutionContext, WTFMove(message.transferredPorts));
-                dispatchEvent(MessageEvent::create(WTFMove(ports), WTFMove(message.message)));
+                dispatchEvent(MessageEvent::create(WTFMove(ports), message.message.releaseNonNull()));
             }
         };
 
-        if (!m_scriptExecutionContext)
-            return;
-
-        if (m_scriptExecutionContext->isContextThread()) {
+        if (!workerThread) {
             innerHandler(WTFMove(messages));
             completionCallback();
             return;
         }
-
-        m_scriptExecutionContext->postTask([innerHandler = WTFMove(innerHandler), messages = WTFMove(messages), completionCallback = WTFMove(completionCallback)](ScriptExecutionContext&) mutable {
+        workerThread->runLoop().postTaskForMode([innerHandler = WTFMove(innerHandler), messages = WTFMove(messages), completionCallback = WTFMove(completionCallback)](auto&) mutable {
             innerHandler(WTFMove(messages));
-            RunLoop::main().dispatch([completionCallback = WTFMove(completionCallback)] {
+            callOnMainThread([completionCallback = WTFMove(completionCallback)] {
                 completionCallback();
             });
-        });
+        }, WorkerRunLoop::defaultMode());
     };
 
     MessagePortChannelProvider::singleton().takeAllMessagesForPort(m_identifier, WTFMove(messagesTakenHandler));
 }
 
+void MessagePort::updateActivity(MessagePortChannelProvider::HasActivity hasActivity)
+{
+    bool hasHadLocalActivity = m_hasHadLocalActivitySinceLastCheck;
+    m_hasHadLocalActivitySinceLastCheck = false;
+
+    if (hasActivity == MessagePortChannelProvider::HasActivity::No && !hasHadLocalActivity)
+        m_isRemoteEligibleForGC = true;
+
+    if (hasActivity == MessagePortChannelProvider::HasActivity::Yes)
+        m_isRemoteEligibleForGC = false;
+
+    m_isAskingRemoteAboutGC = false;
+}
+
 bool MessagePort::hasPendingActivity() const
 {
     m_mightBeEligibleForGC = true;
@@ -305,32 +320,23 @@ bool MessagePort::hasPendingActivity() const
 
     // If we're not in the middle of asking the remote port about collectability, do so now.
     if (!m_isAskingRemoteAboutGC) {
-        MessagePortChannelProvider::singleton().checkRemotePortForActivity(m_remoteIdentifier, [this, protectedThis = makeRef(*this)](MessagePortChannelProvider::HasActivity hasActivity) mutable {
-            auto innerHandler = [this, otherProtectedThis = WTFMove(protectedThis)](MessagePortChannelProvider::HasActivity hasActivity) {
-                bool hasHadLocalActivity = m_hasHadLocalActivitySinceLastCheck;
-                m_hasHadLocalActivitySinceLastCheck = false;
+        RefPtr<WorkerThread> workerThread;
+        if (is<WorkerGlobalScope>(*m_scriptExecutionContext))
+            workerThread = &downcast<WorkerGlobalScope>(*m_scriptExecutionContext).thread();
 
-                if (hasActivity == MessagePortChannelProvider::HasActivity::No && !hasHadLocalActivity)
-                    m_isRemoteEligibleForGC = true;
+        MessagePortChannelProvider::singleton().checkRemotePortForActivity(m_remoteIdentifier, [weakThis = makeWeakPtr(const_cast<MessagePort*>(this)), workerThread = WTFMove(workerThread)](MessagePortChannelProvider::HasActivity hasActivity) mutable {
 
-                if (hasActivity == MessagePortChannelProvider::HasActivity::Yes)
-                    m_isRemoteEligibleForGC = false;
-
-                m_isAskingRemoteAboutGC = false;
-            };
-
-
-            if (!m_scriptExecutionContext)
-                return;
-
-            if (m_scriptExecutionContext->isContextThread()) {
-                innerHandler(hasActivity);
+            ASSERT(isMainThread());
+            if (!workerThread) {
+                if (weakThis)
+                    weakThis->updateActivity(hasActivity);
                 return;
             }
 
-            m_scriptExecutionContext->postTask([innerHandler = WTFMove(innerHandler), hasActivity](ScriptExecutionContext&) mutable {
-                innerHandler(hasActivity);
-            });
+            workerThread->runLoop().postTaskForMode([weakThis = WTFMove(weakThis), hasActivity](auto&) mutable {
+                if (weakThis)
+                    weakThis->updateActivity(hasActivity);
+            }, WorkerRunLoop::defaultMode());
         });
         m_isAskingRemoteAboutGC = true;
     }
@@ -371,7 +377,7 @@ ExceptionOr<TransferredMessagePortArray> MessagePort::disentanglePorts(Vector<Re
         port->disentangle();
     }
 
-    return WTFMove(portArray);
+    return portArray;
 }
 
 Vector<RefPtr<MessagePort>> MessagePort::entanglePorts(ScriptExecutionContext& context, TransferredMessagePortArray&& transferredPorts)
