@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2011 Google Inc. All rights reserved.
+ * Copyright (C) 2018 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -31,8 +32,10 @@
 #pragma once
 
 #include "ExceptionOr.h"
+#include "GCReachableRef.h"
 #include <wtf/Forward.h>
 #include <wtf/HashSet.h>
+#include <wtf/IsoMalloc.h>
 #include <wtf/Vector.h>
 
 namespace WebCore {
@@ -46,7 +49,8 @@ class Node;
 using MutationObserverOptions = unsigned char;
 using MutationRecordDeliveryOptions = unsigned char;
 
-class MutationObserver : public RefCounted<MutationObserver> {
+class MutationObserver final : public RefCounted<MutationObserver> {
+    WTF_MAKE_ISO_ALLOCATED(MutationObserver);
     friend class MutationObserverMicrotask;
 public:
     enum MutationType {
@@ -73,16 +77,21 @@ public:
 
     struct Init {
         bool childList;
-        std::optional<bool> attributes;
-        std::optional<bool> characterData;
+        Optional<bool> attributes;
+        Optional<bool> characterData;
         bool subtree;
-        std::optional<bool> attributeOldValue;
-        std::optional<bool> characterDataOldValue;
-        std::optional<Vector<String>> attributeFilter;
+        Optional<bool> attributeOldValue;
+        Optional<bool> characterDataOldValue;
+        Optional<Vector<String>> attributeFilter;
     };
 
     ExceptionOr<void> observe(Node&, const Init&);
-    Vector<Ref<MutationRecord>> takeRecords();
+    
+    struct TakenRecords {
+        Vector<Ref<MutationRecord>> records;
+        HashSet<GCReachableRef<Node>> pendingTargets;
+    };
+    TakenRecords takeRecords();
     void disconnect();
 
     void observationStarted(MutationObserverRegistration&);
@@ -106,6 +115,7 @@ private:
 
     Ref<MutationCallback> m_callback;
     Vector<Ref<MutationRecord>> m_records;
+    HashSet<GCReachableRef<Node>> m_pendingTargets;
     HashSet<MutationObserverRegistration*> m_registrations;
     unsigned m_priority;
 };
