@@ -26,10 +26,10 @@
 #include "HTMLNames.h"
 #include "LiveNodeList.h"
 #include "MutationObserverRegistration.h"
-#include "Page.h"
 #include "QualifiedName.h"
 #include "TagCollection.h"
 #include <wtf/HashSet.h>
+#include <wtf/StdLibExtras.h>
 #include <wtf/text/AtomicString.h>
 
 namespace WebCore {
@@ -250,6 +250,27 @@ public:
 class NodeRareData : public NodeRareDataBase {
     WTF_MAKE_NONCOPYABLE(NodeRareData); WTF_MAKE_FAST_ALLOCATED;
 public:
+#if defined(DUMP_NODE_STATISTICS) && DUMP_NODE_STATISTICS
+    enum class UseType : uint16_t {
+        ConnectedFrameCount = 1 << 0,
+        NodeList = 1 << 1,
+        MutationObserver = 1 << 2,
+
+        TabIndex = 1 << 3,
+        StyleFlags = 1 << 4,
+        MinimumSize = 1 << 5,
+        ScrollingPosition = 1 << 6,
+        ComputedStyle = 1 << 7,
+        Dataset = 1 << 8,
+        ClassList = 1 << 9,
+        ShadowRoot = 1 << 10,
+        CustomElementQueue = 1 << 11,
+        AttributeMap = 1 << 12,
+        InteractionObserver = 1 << 13,
+        PseudoElements = 1 << 14,
+    };
+#endif
+
     NodeRareData(RenderObject* renderer)
         : NodeRareDataBase(renderer)
         , m_connectedFrameCount(0)
@@ -284,8 +305,22 @@ public:
         m_connectedFrameCount -= amount;
     }
 
+#if DUMP_NODE_STATISTICS
+    OptionSet<UseType> useTypes() const
+    {
+        OptionSet<UseType> result;
+        if (m_connectedFrameCount)
+            result.add(UseType::ConnectedFrameCount);
+        if (m_nodeLists)
+            result.add(UseType::NodeList);
+        if (m_mutationObserverData)
+            result.add(UseType::MutationObserver);
+        return result;
+    }
+#endif
+
 private:
-    unsigned m_connectedFrameCount : 10; // Must fit Page::maxNumberOfFrames.
+    unsigned m_connectedFrameCount; // Must fit Page::maxNumberOfFrames.
 
     std::unique_ptr<NodeListsNodeData> m_nodeLists;
     std::unique_ptr<NodeMutationObserverData> m_mutationObserverData;
@@ -314,7 +349,4 @@ inline NodeRareData& Node::ensureRareData()
     return *rareData();
 }
 
-// Ensure the 10 bits reserved for the m_connectedFrameCount cannot overflow
-static_assert(Page::maxNumberOfFrames < 1024, "Frame limit should fit in rare data count");
-
 } // namespace WebCore
