@@ -34,10 +34,10 @@
 namespace WebCore {
 using namespace JSC;
 
-PromiseRejectionEvent::PromiseRejectionEvent(ExecState& state, const AtomicString& type, const Init& initializer, IsTrusted isTrusted)
+PromiseRejectionEvent::PromiseRejectionEvent(const AtomicString& type, const Init& initializer, IsTrusted isTrusted)
     : Event(type, initializer, isTrusted)
     , m_promise(*(initializer.promise))
-    , m_reason(state.vm(), initializer.reason)
+    , m_reason(initializer.reason)
 {
 }
 
