@@ -55,6 +55,11 @@ public:
         s_count--;
     }
 
+    ScriptDisallowedScope& operator=(const ScriptDisallowedScope&)
+    {
+        return *this;
+    }
+
     static bool isEventAllowedInMainThread()
     {
         return !isMainThread() || !s_count;
