@@ -25,17 +25,12 @@
 
 #pragma once
 
-#include "PlatformScreen.h"
 #include "Timer.h"
 #include <wtf/OptionSet.h>
 #include <wtf/RefCounted.h>
 #include <wtf/RefPtr.h>
 #include <wtf/Vector.h>
 
-#if USE(REQUEST_ANIMATION_FRAME_DISPLAY_MONITOR)
-#include "DisplayRefreshMonitorClient.h"
-#endif
-
 namespace WebCore {
 
 class Document;
@@ -43,14 +38,11 @@ class Page;
 class RequestAnimationFrameCallback;
 
 class ScriptedAnimationController : public RefCounted<ScriptedAnimationController>
-#if USE(REQUEST_ANIMATION_FRAME_DISPLAY_MONITOR)
-    , public DisplayRefreshMonitorClient
-#endif
 {
 public:
-    static Ref<ScriptedAnimationController> create(Document& document, PlatformDisplayID displayID)
+    static Ref<ScriptedAnimationController> create(Document& document)
     {
-        return adoptRef(*new ScriptedAnimationController(document, displayID));
+        return adoptRef(*new ScriptedAnimationController(document));
     }
     ~ScriptedAnimationController();
     void clearDocumentPointer() { m_document = nullptr; }
@@ -73,13 +65,19 @@ public:
     };
     void addThrottlingReason(ThrottlingReason);
     void removeThrottlingReason(ThrottlingReason);
+
     WEBCORE_EXPORT bool isThrottled() const;
     WEBCORE_EXPORT Seconds interval() const;
 
-    void windowScreenDidChange(PlatformDisplayID);
+#if USE(REQUEST_ANIMATION_FRAME_DISPLAY_MONITOR)
+    void documentAnimationSchedulerDidFire();
+#endif
 
 private:
-    ScriptedAnimationController(Document&, PlatformDisplayID);
+    ScriptedAnimationController(Document&);
+
+    void scheduleAnimation();
+    void animationTimerFired();
 
     Page* page() const;
 
@@ -90,19 +88,12 @@ private:
     CallbackId m_nextCallbackId { 0 };
     int m_suspendCount { 0 };
 
-    void scheduleAnimation();
-    void animationTimerFired();
     Timer m_animationTimer;
     double m_lastAnimationFrameTimestamp { 0 };
 
 #if USE(REQUEST_ANIMATION_FRAME_DISPLAY_MONITOR)
-    // Override for DisplayRefreshMonitorClient
-    void displayRefreshFired() override;
-    RefPtr<DisplayRefreshMonitor> createDisplayRefreshMonitor(PlatformDisplayID) const override;
-
-    bool m_isUsingTimer { false };
-
     OptionSet<ThrottlingReason> m_throttlingReasons;
+    bool m_isUsingTimer { false };
 #endif
 };
 
