@@ -26,7 +26,7 @@
 
 #pragma once
 
-#include "DocumentOrderedMap.h"
+#include "TreeScopeOrderedMap.h"
 #include <memory>
 #include <wtf/Forward.h>
 #include <wtf/Vector.h>
@@ -37,6 +37,7 @@ namespace WebCore {
 class ContainerNode;
 class Document;
 class Element;
+class HTMLImageElement;
 class HTMLLabelElement;
 class HTMLMapElement;
 class LayoutPoint;
@@ -75,12 +76,16 @@ public:
     // https://dom.spec.whatwg.org/#retarget
     Node& retargetToScope(Node&) const;
 
-    Node* ancestorNodeInThisScope(Node*) const;
+    WEBCORE_EXPORT Node* ancestorNodeInThisScope(Node*) const;
     WEBCORE_EXPORT Element* ancestorElementInThisScope(Element*) const;
 
     void addImageMap(HTMLMapElement&);
     void removeImageMap(HTMLMapElement&);
-    HTMLMapElement* getImageMap(const String& url) const;
+    HTMLMapElement* getImageMap(const AtomicString&) const;
+
+    void addImageElementByUsemap(const AtomicStringImpl&, HTMLImageElement&);
+    void removeImageElementByUsemap(const AtomicStringImpl&, HTMLImageElement&);
+    HTMLImageElement* imageElementByUsemap(const AtomicStringImpl&) const;
 
     // For accessibility.
     bool shouldCacheLabelsByForAttribute() const { return !!m_labelsByForAttribute; }
@@ -121,10 +126,11 @@ private:
     std::reference_wrapper<Document> m_documentScope;
     TreeScope* m_parentTreeScope;
 
-    std::unique_ptr<DocumentOrderedMap> m_elementsById;
-    std::unique_ptr<DocumentOrderedMap> m_elementsByName;
-    std::unique_ptr<DocumentOrderedMap> m_imageMapsByName;
-    std::unique_ptr<DocumentOrderedMap> m_labelsByForAttribute;
+    std::unique_ptr<TreeScopeOrderedMap> m_elementsById;
+    std::unique_ptr<TreeScopeOrderedMap> m_elementsByName;
+    std::unique_ptr<TreeScopeOrderedMap> m_imageMapsByName;
+    std::unique_ptr<TreeScopeOrderedMap> m_imagesByUsemap;
+    std::unique_ptr<TreeScopeOrderedMap> m_labelsByForAttribute;
 
     std::unique_ptr<IdTargetObserverRegistry> m_idTargetObserverRegistry;
 };
