@@ -49,11 +49,11 @@ int64_t HistoryItem::generateSequenceNumber()
     return ++next;
 }
 
-static void defaultNotifyHistoryItemChanged(HistoryItem*)
+static void defaultNotifyHistoryItemChanged(HistoryItem&)
 {
 }
 
-WEBCORE_EXPORT void (*notifyHistoryItemChanged)(HistoryItem*) = defaultNotifyHistoryItemChanged;
+void (*notifyHistoryItemChanged)(HistoryItem&) = defaultNotifyHistoryItemChanged;
 
 HistoryItem::HistoryItem()
     : HistoryItem({ }, { })
@@ -66,7 +66,7 @@ HistoryItem::HistoryItem(const String& urlString, const String& title)
 }
 
 HistoryItem::HistoryItem(const String& urlString, const String& title, const String& alternateTitle)
-    : HistoryItem(urlString, title, alternateTitle, { Process::identifier(), generateObjectIdentifier<BackForwardItemIdentifier::ItemIdentifierType>() })
+    : HistoryItem(urlString, title, alternateTitle, { Process::identifier(), ObjectIdentifier<BackForwardItemIdentifier::ItemIdentifierType>::generate() })
 {
 }
 
@@ -101,7 +101,7 @@ inline HistoryItem::HistoryItem(const HistoryItem& item)
     , m_documentSequenceNumber(item.m_documentSequenceNumber)
     , m_formContentType(item.m_formContentType)
     , m_pruningReason(PruningReason::None)
-#if PLATFORM(IOS)
+#if PLATFORM(IOS_FAMILY)
     , m_obscuredInsets(item.m_obscuredInsets)
     , m_scale(item.m_scale)
     , m_scaleIsInitial(item.m_scaleIsInitial)
@@ -174,12 +174,12 @@ bool HistoryItem::hasCachedPageExpired() const
 
 URL HistoryItem::url() const
 {
-    return URL(ParsedURLString, m_urlString);
+    return URL({ }, m_urlString);
 }
 
 URL HistoryItem::originalURL() const
 {
-    return URL(ParsedURLString, m_originalURLString);
+    return URL({ }, m_originalURLString);
 }
 
 const String& HistoryItem::referrer() const
@@ -195,13 +195,13 @@ const String& HistoryItem::target() const
 void HistoryItem::setAlternateTitle(const String& alternateTitle)
 {
     m_displayTitle = alternateTitle;
-    notifyHistoryItemChanged(this);
+    notifyChanged();
 }
 
 void HistoryItem::setURLString(const String& urlString)
 {
     m_urlString = urlString;
-    notifyHistoryItemChanged(this);
+    notifyChanged();
 }
 
 void HistoryItem::setURL(const URL& url)
@@ -214,25 +214,25 @@ void HistoryItem::setURL(const URL& url)
 void HistoryItem::setOriginalURLString(const String& urlString)
 {
     m_originalURLString = urlString;
-    notifyHistoryItemChanged(this);
+    notifyChanged();
 }
 
 void HistoryItem::setReferrer(const String& referrer)
 {
     m_referrer = referrer;
-    notifyHistoryItemChanged(this);
+    notifyChanged();
 }
 
 void HistoryItem::setTitle(const String& title)
 {
     m_title = title;
-    notifyHistoryItemChanged(this);
+    notifyChanged();
 }
 
 void HistoryItem::setTarget(const String& target)
 {
     m_target = target;
-    notifyHistoryItemChanged(this);
+    notifyChanged();
 }
 
 const IntPoint& HistoryItem::scrollPosition() const
@@ -258,7 +258,7 @@ bool HistoryItem::shouldRestoreScrollPosition() const
 void HistoryItem::setShouldRestoreScrollPosition(bool shouldRestore)
 {
     m_shouldRestoreScrollPosition = shouldRestore;
-    notifyHistoryItemChanged(this);
+    notifyChanged();
 }
 
 float HistoryItem::pageScaleFactor() const
@@ -309,7 +309,7 @@ void HistoryItem::setIsTargetItem(bool flag)
 void HistoryItem::setStateObject(RefPtr<SerializedScriptValue>&& object)
 {
     m_stateObject = WTFMove(object);
-    notifyHistoryItemChanged(this);
+    notifyChanged();
 }
 
 void HistoryItem::addChildItem(Ref<HistoryItem>&& child)
@@ -372,6 +372,7 @@ void HistoryItem::clearChildren()
 // - The other item corresponds to the same set of documents, including frames (for history entries created via regular navigation)
 bool HistoryItem::shouldDoSameDocumentNavigationTo(HistoryItem& otherItem) const
 {
+    // The following logic must be kept in sync with WebKit::WebBackForwardListItem::itemIsInSameDocument().
     if (this == &otherItem)
         return false;
 
@@ -465,7 +466,7 @@ bool HistoryItem::isCurrentDocument(Document& document) const
 
 void HistoryItem::notifyChanged()
 {
-    notifyHistoryItemChanged(this);
+    notifyHistoryItemChanged(*this);
 }
 
 #ifndef NDEBUG
