@@ -56,6 +56,7 @@
 #include "CSSPropertyNames.h"
 #include "CommonVM.h"
 #include "CookieJar.h"
+#include "DOMWindow.h"
 #include "DocumentLoader.h"
 #include "DocumentType.h"
 #include "ElementChildIterator.h"
@@ -76,10 +77,13 @@
 #include "HashTools.h"
 #include "ScriptController.h"
 #include "StyleResolver.h"
+#include <wtf/IsoMallocInlines.h>
 #include <wtf/text/CString.h>
 
 namespace WebCore {
 
+WTF_MAKE_ISO_ALLOCATED_IMPL(HTMLDocument);
+
 using namespace HTMLNames;
 
 HTMLDocument::HTMLDocument(Frame* frame, const URL& url, DocumentClassFlags documentClasses, unsigned constructionFlags)
@@ -88,21 +92,19 @@ HTMLDocument::HTMLDocument(Frame* frame, const URL& url, DocumentClassFlags docu
     clearXMLVersion();
 }
 
-HTMLDocument::~HTMLDocument()
-{
-}
+HTMLDocument::~HTMLDocument() = default;
 
 int HTMLDocument::width()
 {
     updateLayoutIgnorePendingStylesheets();
-    FrameView* frameView = view();
+    RefPtr<FrameView> frameView = view();
     return frameView ? frameView->contentsWidth() : 0;
 }
 
 int HTMLDocument::height()
 {
     updateLayoutIgnorePendingStylesheets();
-    FrameView* frameView = view();
+    RefPtr<FrameView> frameView = view();
     return frameView ? frameView->contentsHeight() : 0;
 }
 
@@ -112,24 +114,24 @@ Ref<DocumentParser> HTMLDocument::createParser()
 }
 
 // https://html.spec.whatwg.org/multipage/dom.html#dom-document-nameditem
-std::optional<Variant<RefPtr<DOMWindow>, RefPtr<Element>, RefPtr<HTMLCollection>>> HTMLDocument::namedItem(const AtomicString& name)
+Optional<Variant<RefPtr<WindowProxy>, RefPtr<Element>, RefPtr<HTMLCollection>>> HTMLDocument::namedItem(const AtomicString& name)
 {
     if (name.isNull() || !hasDocumentNamedItem(*name.impl()))
-        return std::nullopt;
+        return WTF::nullopt;
 
     if (UNLIKELY(documentNamedItemContainsMultipleElements(*name.impl()))) {
         auto collection = documentNamedItems(name);
         ASSERT(collection->length() > 1);
-        return Variant<RefPtr<DOMWindow>, RefPtr<Element>, RefPtr<HTMLCollection>> { RefPtr<HTMLCollection> { WTFMove(collection) } };
+        return Variant<RefPtr<WindowProxy>, RefPtr<Element>, RefPtr<HTMLCollection>> { RefPtr<HTMLCollection> { WTFMove(collection) } };
     }
 
     auto& element = *documentNamedItem(*name.impl());
     if (UNLIKELY(is<HTMLIFrameElement>(element))) {
-        if (auto* domWindow = downcast<HTMLIFrameElement>(element).contentWindow())
-            return Variant<RefPtr<DOMWindow>, RefPtr<Element>, RefPtr<HTMLCollection>> { RefPtr<DOMWindow> { domWindow } };
+        if (auto domWindow = makeRefPtr(downcast<HTMLIFrameElement>(element).contentWindow()))
+            return Variant<RefPtr<WindowProxy>, RefPtr<Element>, RefPtr<HTMLCollection>> { WTFMove(domWindow) };
     }
 
-    return Variant<RefPtr<DOMWindow>, RefPtr<Element>, RefPtr<HTMLCollection>> { RefPtr<Element> { &element } };
+    return Variant<RefPtr<WindowProxy>, RefPtr<Element>, RefPtr<HTMLCollection>> { RefPtr<Element> { &element } };
 }
 
 Vector<AtomicString> HTMLDocument::supportedPropertyNames() const
@@ -179,51 +181,51 @@ bool HTMLDocument::isCaseSensitiveAttribute(const QualifiedName& attributeName)
         // This is the list of attributes in HTML 4.01 with values marked as "[CI]" or case-insensitive
         // Mozilla treats all other values as case-sensitive, thus so do we.
         static const QualifiedName* const names[] = {
-            &accept_charsetAttr,
-            &acceptAttr,
-            &alignAttr,
-            &alinkAttr,
-            &axisAttr,
-            &bgcolorAttr,
-            &charsetAttr,
-            &checkedAttr,
-            &clearAttr,
-            &codetypeAttr,
-            &colorAttr,
-            &compactAttr,
-            &declareAttr,
-            &deferAttr,
-            &dirAttr,
-            &disabledAttr,
-            &enctypeAttr,
-            &faceAttr,
-            &frameAttr,
-            &hreflangAttr,
-            &http_equivAttr,
-            &langAttr,
-            &languageAttr,
-            &linkAttr,
-            &mediaAttr,
-            &methodAttr,
-            &multipleAttr,
-            &nohrefAttr,
-            &noresizeAttr,
-            &noshadeAttr,
-            &nowrapAttr,
-            &readonlyAttr,
-            &relAttr,
-            &revAttr,
-            &rulesAttr,
-            &scopeAttr,
-            &scrollingAttr,
-            &selectedAttr,
-            &shapeAttr,
-            &targetAttr,
-            &textAttr,
-            &typeAttr,
-            &valignAttr,
-            &valuetypeAttr,
-            &vlinkAttr,
+            &accept_charsetAttr.get(),
+            &acceptAttr.get(),
+            &alignAttr.get(),
+            &alinkAttr.get(),
+            &axisAttr.get(),
+            &bgcolorAttr.get(),
+            &charsetAttr.get(),
+            &checkedAttr.get(),
+            &clearAttr.get(),
+            &codetypeAttr.get(),
+            &colorAttr.get(),
+            &compactAttr.get(),
+            &declareAttr.get(),
+            &deferAttr.get(),
+            &dirAttr.get(),
+            &disabledAttr.get(),
+            &enctypeAttr.get(),
+            &faceAttr.get(),
+            &frameAttr.get(),
+            &hreflangAttr.get(),
+            &http_equivAttr.get(),
+            &langAttr.get(),
+            &languageAttr.get(),
+            &linkAttr.get(),
+            &mediaAttr.get(),
+            &methodAttr.get(),
+            &multipleAttr.get(),
+            &nohrefAttr.get(),
+            &noresizeAttr.get(),
+            &noshadeAttr.get(),
+            &nowrapAttr.get(),
+            &readonlyAttr.get(),
+            &relAttr.get(),
+            &revAttr.get(),
+            &rulesAttr.get(),
+            &scopeAttr.get(),
+            &scrollingAttr.get(),
+            &selectedAttr.get(),
+            &shapeAttr.get(),
+            &targetAttr.get(),
+            &textAttr.get(),
+            &typeAttr.get(),
+            &valignAttr.get(),
+            &valuetypeAttr.get(),
+            &vlinkAttr.get(),
         };
         HashSet<AtomicString> set;
         for (auto* name : names)
