@@ -464,27 +464,27 @@ bool HTMLPlugInElement::isReplacementObscured()
     HitTestLocation location = LayoutPoint(x + width / 2, y + height / 2);
     ASSERT(!renderView->needsLayout());
     ASSERT(!renderView->document().needsStyleRecalc());
-    bool hit = renderView->hitTest(request, location, result);
+    bool hit = topDocument->hitTest(request, location, result);
     if (!hit || result.innerNode() != &pluginRenderer.frameOwnerElement())
         return true;
 
     location = LayoutPoint(x, y);
-    hit = renderView->hitTest(request, location, result);
+    hit = topDocument->hitTest(request, location, result);
     if (!hit || result.innerNode() != &pluginRenderer.frameOwnerElement())
         return true;
 
     location = LayoutPoint(x + width, y);
-    hit = renderView->hitTest(request, location, result);
+    hit = topDocument->hitTest(request, location, result);
     if (!hit || result.innerNode() != &pluginRenderer.frameOwnerElement())
         return true;
 
     location = LayoutPoint(x + width, y + height);
-    hit = renderView->hitTest(request, location, result);
+    hit = topDocument->hitTest(request, location, result);
     if (!hit || result.innerNode() != &pluginRenderer.frameOwnerElement())
         return true;
 
     location = LayoutPoint(x, y + height);
-    hit = renderView->hitTest(request, location, result);
+    hit = topDocument->hitTest(request, location, result);
     if (!hit || result.innerNode() != &pluginRenderer.frameOwnerElement())
         return true;
     return false;
