@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007, 2008, 2010, 2014 Apple Inc. All rights reserved.
+ * Copyright (C) 2006-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -398,6 +398,9 @@ void ImageDocument::imageClicked(int x, int y)
 
         updateLayout();
 
+        if (!view())
+            return;
+
         float scale = this->scale();
 
         IntSize viewportSize = view()->visibleSize();
