@@ -32,12 +32,11 @@
 #include "HTMLMediaElement.h"
 #include "TimeRanges.h"
 #include <pal/system/Clock.h>
-#include <wtf/CurrentTime.h>
 #include <wtf/NeverDestroyed.h>
 #include <wtf/StdLibExtras.h>
 #include <wtf/text/AtomicString.h>
 
-using namespace WebCore;
+namespace WebCore {
 
 Ref<MediaController> MediaController::create(ScriptExecutionContext& context)
 {
@@ -533,7 +532,7 @@ bool MediaController::hasEnded() const
 
 void MediaController::scheduleEvent(const AtomicString& eventName)
 {
-    m_pendingEvents.append(Event::create(eventName, false, true));
+    m_pendingEvents.append(Event::create(eventName, Event::CanBubble::No, Event::IsCancelable::Yes));
     if (!m_asyncEventTimer.isActive())
         m_asyncEventTimer.startOneShot(0_s);
 }
@@ -683,4 +682,6 @@ void MediaController::scheduleTimeupdateEvent()
     m_previousTimeupdateTime = now;
 }
 
+} // namespace WebCore
+
 #endif
