@@ -39,7 +39,6 @@
 
 namespace WebCore {
 
-// Must match WebKitSystemInterface.h
 enum MediaControlElementType {
     MediaEnterFullscreenButton = 0,
     MediaMuteButton,
@@ -73,8 +72,8 @@ enum MediaControlElementType {
     MediaClosedCaptionsTrackList,
 };
 
-HTMLMediaElement* parentMediaElement(Node*);
-inline HTMLMediaElement* parentMediaElement(const RenderObject& renderer) { return parentMediaElement(renderer.node()); }
+RefPtr<HTMLMediaElement> parentMediaElement(Node*);
+inline RefPtr<HTMLMediaElement> parentMediaElement(const RenderObject& renderer) { return parentMediaElement(renderer.node()); }
 
 MediaControlElementType mediaControlElementType(Node*);
 
@@ -93,7 +92,7 @@ public:
 
 protected:
     explicit MediaControlElement(MediaControlElementType, HTMLElement*);
-    ~MediaControlElement() { }
+    ~MediaControlElement() = default;
 
     virtual void setDisplayType(MediaControlElementType);
     virtual bool isMediaControlElement() const { return true; }
@@ -107,6 +106,7 @@ private:
 // ----------------------------
 
 class MediaControlDivElement : public HTMLDivElement, public MediaControlElement {
+    WTF_MAKE_ISO_ALLOCATED(MediaControlDivElement);
 protected:
     explicit MediaControlDivElement(Document&, MediaControlElementType);
 
@@ -117,6 +117,7 @@ private:
 // ----------------------------
 
 class MediaControlInputElement : public HTMLInputElement, public MediaControlElement {
+    WTF_MAKE_ISO_ALLOCATED(MediaControlInputElement);
 protected:
     explicit MediaControlInputElement(Document&, MediaControlElementType);
 
@@ -128,6 +129,7 @@ private:
 // ----------------------------
 
 class MediaControlTimeDisplayElement : public MediaControlDivElement {
+    WTF_MAKE_ISO_ALLOCATED(MediaControlTimeDisplayElement);
 public:
     void setCurrentValue(double);
     double currentValue() const { return m_currentValue; }
@@ -142,6 +144,7 @@ private:
 // ----------------------------
 
 class MediaControlMuteButtonElement : public MediaControlInputElement {
+    WTF_MAKE_ISO_ALLOCATED(MediaControlMuteButtonElement);
 public:
     void changedMute();
 
@@ -159,6 +162,7 @@ private:
 // ----------------------------
 
 class MediaControlSeekButtonElement : public MediaControlInputElement {
+    WTF_MAKE_ISO_ALLOCATED(MediaControlSeekButtonElement);
 public:
     bool willRespondToMouseClickEvents() override { return true; }
 
@@ -175,6 +179,7 @@ private:
 // ----------------------------
 
 class MediaControlVolumeSliderElement : public MediaControlInputElement {
+    WTF_MAKE_ISO_ALLOCATED(MediaControlVolumeSliderElement);
 public:
     bool willRespondToMouseMoveEvents() override;
     bool willRespondToMouseClickEvents() override;
