@@ -30,6 +30,7 @@
 
 #pragma once
 
+#include "CertificateInfo.h"
 #include "UserInterfaceLayoutDirection.h"
 #include <wtf/Forward.h>
 #include <wtf/text/WTFString.h>
@@ -53,13 +54,15 @@ public:
     virtual void startWindowDrag() = 0;
     virtual void moveWindowBy(float x, float y) = 0;
 
+    virtual bool isRemote() const = 0;
     virtual String localizedStringsURL() = 0;
     virtual unsigned inspectionLevel() const = 0;
     virtual String backendCommandsURL() { return String(); };
-    virtual String debuggableType() { return ASCIILiteral("web"); }
+    virtual String debuggableType() { return "web"_s; }
 
     virtual void bringToFront() = 0;
     virtual void closeWindow() = 0;
+    virtual void reopen() = 0;
 
     virtual UserInterfaceLayoutDirection userInterfaceLayoutDirection() const = 0;
 
@@ -74,6 +77,7 @@ public:
     virtual void append(const WTF::String& url, const WTF::String& content) = 0;
 
     virtual void inspectedURLChanged(const String&) = 0;
+    virtual void showCertificate(const CertificateInfo&) = 0;
 
     virtual void pagePaused() { }
     virtual void pageUnpaused() { }
