@@ -27,9 +27,9 @@
 #pragma once
 
 #include "StoredCredentialsPolicy.h"
-#include "URLHash.h"
 #include <wtf/HashMap.h>
 #include <wtf/HashSet.h>
+#include <wtf/URLHash.h>
 #include <wtf/text/StringHash.h>
 
 namespace WebCore {
@@ -45,7 +45,7 @@ public:
     {
     }
 
-    WEBCORE_EXPORT bool parse(const ResourceResponse&, String& errorDescription);
+    WEBCORE_EXPORT bool parse(const ResourceResponse&);
     WEBCORE_EXPORT bool allowsCrossOriginMethod(const String&, String& errorDescription) const;
     WEBCORE_EXPORT bool allowsCrossOriginHeaders(const HTTPHeaderMap&, String& errorDescription) const;
     bool allowsRequest(StoredCredentialsPolicy, const String& method, const HTTPHeaderMap& requestHeaders) const;
@@ -62,14 +62,11 @@ private:
 
 class CrossOriginPreflightResultCache {
     WTF_MAKE_NONCOPYABLE(CrossOriginPreflightResultCache); WTF_MAKE_FAST_ALLOCATED;
-
 public:
     WEBCORE_EXPORT static CrossOriginPreflightResultCache& singleton();
-
     WEBCORE_EXPORT void appendEntry(const String& origin, const URL&, std::unique_ptr<CrossOriginPreflightResultCacheItem>);
     WEBCORE_EXPORT bool canSkipPreflight(const String& origin, const URL&, StoredCredentialsPolicy, const String& method, const HTTPHeaderMap& requestHeaders);
-
-    WEBCORE_EXPORT void empty();
+    WEBCORE_EXPORT void clear();
 
 private:
     friend NeverDestroyed<CrossOriginPreflightResultCache>;
