@@ -36,15 +36,13 @@
 
 namespace WebCore {
 
-CachedScript::CachedScript(CachedResourceRequest&& request, SessionID sessionID)
-    : CachedResource(WTFMove(request), Script, sessionID)
-    , m_decoder(TextResourceDecoder::create(ASCIILiteral("application/javascript"), request.charset()))
+CachedScript::CachedScript(CachedResourceRequest&& request, const PAL::SessionID& sessionID, const CookieJar* cookieJar)
+    : CachedResource(WTFMove(request), Type::Script, sessionID, cookieJar)
+    , m_decoder(TextResourceDecoder::create("text/javascript"_s, request.charset()))
 {
 }
 
-CachedScript::~CachedScript()
-{
-}
+CachedScript::~CachedScript() = default;
 
 void CachedScript::setEncoding(const String& chs)
 {
