@@ -43,6 +43,12 @@ RuntimeEnabledFeatures::RuntimeEnabledFeatures()
 #if ENABLE(MEDIA_STREAM) && PLATFORM(COCOA)
     m_isMediaDevicesEnabled = false;
 #endif
+#if PLATFORM(WATCHOS)
+    m_isWebSocketEnabled = false;
+#endif
+#if PLATFORM(GTK) && ENABLE(INPUT_TYPE_COLOR)
+    m_isInputTypeColorEnabled = true;
+#endif
 }
 
 RuntimeEnabledFeatures& RuntimeEnabledFeatures::sharedFeatures()
@@ -57,11 +63,4 @@ bool RuntimeEnabledFeatures::spectreGadgetsEnabled() const
     return JSC::Options::enableSpectreGadgets();
 }
 
-#if ENABLE(VIDEO)
-bool RuntimeEnabledFeatures::audioEnabled() const
-{
-    return MediaPlayer::isAvailable();
-}
-#endif
-
 } // namespace WebCore
