@@ -29,23 +29,19 @@
 #if ENABLE(ASYNC_SCROLLING) || USE(COORDINATED_GRAPHICS)
 
 #include "AsyncScrollingCoordinator.h"
+#include "Logging.h"
 #include "ScrollingStateFixedNode.h"
+#include "ScrollingStateFrameHostingNode.h"
 #include "ScrollingStateFrameScrollingNode.h"
 #include "ScrollingStateOverflowScrollingNode.h"
 #include "ScrollingStateStickyNode.h"
 #include <wtf/text/CString.h>
-
-#ifndef NDEBUG
-#include <stdio.h>
-#endif
+#include <wtf/text/TextStream.h>
 
 namespace WebCore {
 
 ScrollingStateTree::ScrollingStateTree(AsyncScrollingCoordinator* scrollingCoordinator)
     : m_scrollingCoordinator(scrollingCoordinator)
-    , m_hasChangedProperties(false)
-    , m_hasNewRootStateNode(false)
-    , m_preferredLayerRepresentation(LayerRepresentation::GraphicsLayerRepresentation)
 {
 }
 
@@ -68,92 +64,199 @@ void ScrollingStateTree::setHasChangedProperties(bool changedProperties)
 Ref<ScrollingStateNode> ScrollingStateTree::createNode(ScrollingNodeType nodeType, ScrollingNodeID nodeID)
 {
     switch (nodeType) {
-    case FixedNode:
-        return ScrollingStateFixedNode::create(*this, nodeID);
-    case StickyNode:
-        return ScrollingStateStickyNode::create(*this, nodeID);
-    case FrameScrollingNode:
-        return ScrollingStateFrameScrollingNode::create(*this, nodeID);
-    case OverflowScrollingNode:
+    case ScrollingNodeType::MainFrame:
+    case ScrollingNodeType::Subframe:
+        return ScrollingStateFrameScrollingNode::create(*this, nodeType, nodeID);
+    case ScrollingNodeType::FrameHosting:
+        return ScrollingStateFrameHostingNode::create(*this, nodeID);
+    case ScrollingNodeType::Overflow:
         return ScrollingStateOverflowScrollingNode::create(*this, nodeID);
+    case ScrollingNodeType::Fixed:
+        return ScrollingStateFixedNode::create(*this, nodeID);
+    case ScrollingNodeType::Sticky:
+        return ScrollingStateStickyNode::create(*this, nodeID);
     }
     ASSERT_NOT_REACHED();
     return ScrollingStateFixedNode::create(*this, nodeID);
 }
 
-bool ScrollingStateTree::nodeTypeAndParentMatch(ScrollingStateNode& node, ScrollingNodeType nodeType, ScrollingNodeID parentID) const
+bool ScrollingStateTree::nodeTypeAndParentMatch(ScrollingStateNode& node, ScrollingNodeType nodeType, ScrollingStateNode* parentNode) const
 {
     if (node.nodeType() != nodeType)
         return false;
 
-    auto* parent = stateNodeForID(parentID);
-    if (!parent)
-        return true;
-
-    return node.parent() == parent;
+    return node.parent() == parentNode;
 }
 
-ScrollingNodeID ScrollingStateTree::attachNode(ScrollingNodeType nodeType, ScrollingNodeID newNodeID, ScrollingNodeID parentID)
+ScrollingNodeID ScrollingStateTree::createUnparentedNode(ScrollingNodeType nodeType, ScrollingNodeID newNodeID)
 {
+    LOG_WITH_STREAM(Scrolling, stream << "ScrollingStateTree " << this << " createUnparentedNode " << newNodeID);
+
+    if (auto* node = stateNodeForID(newNodeID)) {
+        if (node->nodeType() == nodeType) {
+            // If the node exists and is parented, unparent it. It may already be in m_unparentedNodes.
+            unparentNode(newNodeID);
+            return newNodeID;
+        }
+
+#if ENABLE(ASYNC_SCROLLING)
+        // If the type has changed, we need to destroy and recreate the node with a new ID.
+        if (nodeType != node->nodeType()) {
+            unparentChildrenAndDestroyNode(newNodeID);
+            newNodeID = m_scrollingCoordinator->uniqueScrollingNodeID();
+        }
+#endif
+    }
+
+    auto stateNode = createNode(nodeType, newNodeID);
+    addNode(stateNode.get());
+    m_unparentedNodes.add(newNodeID, WTFMove(stateNode));
+    return newNodeID;
+}
+
+ScrollingNodeID ScrollingStateTree::insertNode(ScrollingNodeType nodeType, ScrollingNodeID newNodeID, ScrollingNodeID parentID, size_t childIndex)
+{
+    LOG_WITH_STREAM(Scrolling, stream << "ScrollingStateTree " << this << " insertNode " << newNodeID << " in parent " << parentID << " at " << childIndex);
     ASSERT(newNodeID);
 
     if (auto* node = stateNodeForID(newNodeID)) {
-        if (nodeTypeAndParentMatch(*node, nodeType, parentID))
+        auto* parent = stateNodeForID(parentID);
+        if (nodeTypeAndParentMatch(*node, nodeType, parent)) {
+            if (!parentID)
+                return newNodeID;
+
+            size_t currentIndex = parent->indexOfChild(*node);
+            if (currentIndex == childIndex)
+                return newNodeID;
+
+            ASSERT(currentIndex != notFound);
+            Ref<ScrollingStateNode> protectedNode(*node);
+            parent->removeChildAtIndex(currentIndex);
+
+            if (childIndex == notFound)
+                parent->appendChild(WTFMove(protectedNode));
+            else
+                parent->insertChild(WTFMove(protectedNode), childIndex);
+
             return newNodeID;
+        }
 
 #if ENABLE(ASYNC_SCROLLING)
         // If the type has changed, we need to destroy and recreate the node with a new ID.
         if (nodeType != node->nodeType())
-            newNodeID = m_scrollingCoordinator->uniqueScrollLayerID();
+            newNodeID = m_scrollingCoordinator->uniqueScrollingNodeID();
 #endif
 
         // The node is being re-parented. To do that, we'll remove it, and then create a new node.
-        removeNodeAndAllDescendants(node, SubframeNodeRemoval::Orphan);
+        unparentNode(newNodeID);
     }
 
     ScrollingStateNode* newNode = nullptr;
     if (!parentID) {
+        RELEASE_ASSERT(nodeType == ScrollingNodeType::MainFrame);
+        ASSERT(!childIndex || childIndex == notFound);
         // If we're resetting the root node, we should clear the HashMap and destroy the current children.
         clear();
 
-        setRootStateNode(ScrollingStateFrameScrollingNode::create(*this, newNodeID));
+        setRootStateNode(ScrollingStateFrameScrollingNode::create(*this, ScrollingNodeType::MainFrame, newNodeID));
         newNode = rootStateNode();
         m_hasNewRootStateNode = true;
     } else {
         auto* parent = stateNodeForID(parentID);
-        if (!parent)
+        if (!parent) {
+            ASSERT_NOT_REACHED();
             return 0;
+        }
 
-        if (nodeType == FrameScrollingNode && parentID) {
-            if (auto orphanedNode = m_orphanedSubframeNodes.take(newNodeID)) {
-                newNode = orphanedNode.get();
-                parent->appendChild(orphanedNode.releaseNonNull());
+        if (parentID) {
+            if (auto unparentedNode = m_unparentedNodes.take(newNodeID)) {
+                LOG_WITH_STREAM(Scrolling, stream << "ScrollingStateTree " << this << " insertNode " << newNodeID << " getting node from unparented nodes");
+                newNode = unparentedNode.get();
+                nodeWasReattachedRecursive(*unparentedNode);
+
+                if (childIndex == notFound)
+                    parent->appendChild(unparentedNode.releaseNonNull());
+                else
+                    parent->insertChild(unparentedNode.releaseNonNull(), childIndex);
             }
         }
 
         if (!newNode) {
             auto stateNode = createNode(nodeType, newNodeID);
             newNode = stateNode.ptr();
-            parent->appendChild(WTFMove(stateNode));
+            if (childIndex == notFound)
+                parent->appendChild(WTFMove(stateNode));
+            else
+                parent->insertChild(WTFMove(stateNode), childIndex);
         }
     }
 
-    m_stateNodeMap.set(newNodeID, newNode);
-    m_nodesRemovedSinceLastCommit.remove(newNodeID);
+    addNode(*newNode);
     return newNodeID;
 }
 
-void ScrollingStateTree::detachNode(ScrollingNodeID nodeID)
+void ScrollingStateTree::unparentNode(ScrollingNodeID nodeID)
 {
     if (!nodeID)
         return;
 
-    // The node may not be found if clearStateTree() was recently called.
+    LOG_WITH_STREAM(Scrolling, stream << "ScrollingStateTree " << this << " unparentNode " << nodeID);
+
+    // The node may not be found if clear() was recently called.
+    RefPtr<ScrollingStateNode> protectedNode = m_stateNodeMap.get(nodeID);
+    if (!protectedNode)
+        return;
+
+    if (protectedNode == m_rootStateNode)
+        m_rootStateNode = nullptr;
+
+    protectedNode->removeFromParent();
+    m_unparentedNodes.add(nodeID, WTFMove(protectedNode));
+}
+
+void ScrollingStateTree::unparentChildrenAndDestroyNode(ScrollingNodeID nodeID)
+{
+    if (!nodeID)
+        return;
+
+    LOG_WITH_STREAM(Scrolling, stream << "ScrollingStateTree " << this << " unparentChildrenAndDestroyNode " << nodeID);
+
+    // The node may not be found if clear() was recently called.
+    RefPtr<ScrollingStateNode> protectedNode = m_stateNodeMap.take(nodeID);
+    if (!protectedNode)
+        return;
+
+    if (protectedNode == m_rootStateNode)
+        m_rootStateNode = nullptr;
+
+    if (protectedNode->children()) {
+        auto isolatedChildren = protectedNode->takeChildren();
+        for (auto child : *isolatedChildren) {
+            child->removeFromParent();
+            LOG_WITH_STREAM(Scrolling, stream << " moving " << child->scrollingNodeID() << " to unparented nodes");
+            m_unparentedNodes.add(child->scrollingNodeID(), WTFMove(child));
+        }
+    }
+    
+    protectedNode->removeFromParent();
+    willRemoveNode(protectedNode.get());
+}
+
+void ScrollingStateTree::detachAndDestroySubtree(ScrollingNodeID nodeID)
+{
+    if (!nodeID)
+        return;
+
+    LOG_WITH_STREAM(Scrolling, stream << "ScrollingStateTree " << this << " detachAndDestroySubtree " << nodeID);
+
+    // The node may not be found if clear() was recently called.
     auto* node = m_stateNodeMap.take(nodeID);
     if (!node)
         return;
 
-    removeNodeAndAllDescendants(node, SubframeNodeRemoval::Orphan);
+    // If the node was unparented, remove it from m_unparentedNodes (keeping it alive until this function returns).
+    auto unparentedNode = m_unparentedNodes.take(nodeID);
+    removeNodeAndAllDescendants(node);
 }
 
 void ScrollingStateTree::clear()
@@ -162,17 +265,25 @@ void ScrollingStateTree::clear()
         removeNodeAndAllDescendants(rootStateNode());
 
     m_stateNodeMap.clear();
-    m_orphanedSubframeNodes.clear();
+    m_unparentedNodes.clear();
+}
+
+void ScrollingStateTree::nodeWasReattachedRecursive(ScrollingStateNode& node)
+{
+    // When a node is re-attached, the ScrollingTree is recreating the ScrollingNode from scratch, so we need to set all the dirty bits.
+    node.setAllPropertiesChanged();
+
+    if (auto* children = node.children()) {
+        for (auto& child : *children)
+            nodeWasReattachedRecursive(*child);
+    }
 }
 
 std::unique_ptr<ScrollingStateTree> ScrollingStateTree::commit(LayerRepresentation::Type preferredLayerRepresentation)
 {
-    if (!m_orphanedSubframeNodes.isEmpty()) {
-        // If we still have orphaned subtrees, remove them from m_stateNodeMap since they will be deleted 
-        // when clearing m_orphanedSubframeNodes.
-        for (auto& orphanNode : m_orphanedSubframeNodes.values())
-            recursiveNodeWillBeRemoved(orphanNode.get(), SubframeNodeRemoval::Delete);
-        m_orphanedSubframeNodes.clear();
+    if (!m_unparentedNodes.isEmpty()) {
+        // We expect temporarily to have unparented nodes when committing when connecting across iframe boundaries, but unparented nodes should not stick around for a long time.
+        LOG(Scrolling, "Committing with %u unparented nodes", m_unparentedNodes.size());
     }
 
     // This function clones and resets the current state tree, but leaves the tree structure intact.
@@ -182,9 +293,6 @@ std::unique_ptr<ScrollingStateTree> ScrollingStateTree::commit(LayerRepresentati
     if (m_rootStateNode)
         treeStateClone->setRootStateNode(static_reference_cast<ScrollingStateFrameScrollingNode>(m_rootStateNode->cloneAndReset(*treeStateClone)));
 
-    // Copy the IDs of the nodes that have been removed since the last commit into the clone.
-    treeStateClone->m_nodesRemovedSinceLastCommit.swap(m_nodesRemovedSinceLastCommit);
-
     // Now the clone tree has changed properties, and the original tree does not.
     treeStateClone->m_hasChangedProperties = m_hasChangedProperties;
     m_hasChangedProperties = false;
@@ -195,23 +303,28 @@ std::unique_ptr<ScrollingStateTree> ScrollingStateTree::commit(LayerRepresentati
     return treeStateClone;
 }
 
-void ScrollingStateTree::addNode(ScrollingStateNode* node)
+void ScrollingStateTree::setRootStateNode(Ref<ScrollingStateFrameScrollingNode>&& rootStateNode)
 {
-    m_stateNodeMap.add(node->scrollingNodeID(), node);
+    m_rootStateNode = WTFMove(rootStateNode);
 }
 
-void ScrollingStateTree::removeNodeAndAllDescendants(ScrollingStateNode* node, SubframeNodeRemoval subframeNodeRemoval)
+void ScrollingStateTree::addNode(ScrollingStateNode& node)
+{
+    m_stateNodeMap.add(node.scrollingNodeID(), &node);
+}
+
+void ScrollingStateTree::removeNodeAndAllDescendants(ScrollingStateNode* node)
 {
     auto* parent = node->parent();
 
-    recursiveNodeWillBeRemoved(node, subframeNodeRemoval);
+    recursiveNodeWillBeRemoved(node);
 
     if (node == m_rootStateNode)
         m_rootStateNode = nullptr;
     else if (parent) {
         ASSERT(parent->children());
         ASSERT(parent->children()->find(node) != notFound);
-        if (auto children = parent->children()) {
+        if (auto* children = parent->children()) {
             size_t index = children->find(node);
             if (index != notFound)
                 children->remove(index);
@@ -219,34 +332,23 @@ void ScrollingStateTree::removeNodeAndAllDescendants(ScrollingStateNode* node, S
     }
 }
 
-void ScrollingStateTree::recursiveNodeWillBeRemoved(ScrollingStateNode* currNode, SubframeNodeRemoval subframeNodeRemoval)
+void ScrollingStateTree::recursiveNodeWillBeRemoved(ScrollingStateNode* currNode)
 {
     currNode->setParent(nullptr);
-    if (subframeNodeRemoval == SubframeNodeRemoval::Orphan && currNode != m_rootStateNode && currNode->isFrameScrollingNode()) {
-        m_orphanedSubframeNodes.add(currNode->scrollingNodeID(), currNode);
-        return;
-    }
-
     willRemoveNode(currNode);
 
-    if (auto children = currNode->children()) {
+    if (auto* children = currNode->children()) {
         for (auto& child : *children)
-            recursiveNodeWillBeRemoved(child.get(), subframeNodeRemoval);
+            recursiveNodeWillBeRemoved(child.get());
     }
 }
 
 void ScrollingStateTree::willRemoveNode(ScrollingStateNode* node)
 {
-    m_nodesRemovedSinceLastCommit.add(node->scrollingNodeID());
     m_stateNodeMap.remove(node->scrollingNodeID());
     setHasChangedProperties();
 }
 
-void ScrollingStateTree::setRemovedNodes(HashSet<ScrollingNodeID> nodes)
-{
-    m_nodesRemovedSinceLastCommit = WTFMove(nodes);
-}
-
 ScrollingStateNode* ScrollingStateTree::stateNodeForID(ScrollingNodeID scrollLayerID) const
 {
     if (!scrollLayerID)
@@ -270,12 +372,12 @@ void showScrollingStateTree(const WebCore::ScrollingStateTree* tree)
 
     auto rootNode = tree->rootStateNode();
     if (!rootNode) {
-        fprintf(stderr, "Scrolling state tree %p with no root node\n", tree);
+        WTFLogAlways("Scrolling state tree %p with no root node\n", tree);
         return;
     }
 
     String output = rootNode->scrollingStateTreeAsText(WebCore::ScrollingStateTreeAsTextBehaviorDebug);
-    fprintf(stderr, "%s\n", output.utf8().data());
+    WTFLogAlways("%s\n", output.utf8().data());
 }
 
 void showScrollingStateTree(const WebCore::ScrollingStateNode* node)
