@@ -26,8 +26,8 @@
 
 #pragma once
 
-#include "FileSystem.h"
 #include <JavaScriptCore/ArrayBuffer.h>
+#include <wtf/FileSystem.h>
 #include <wtf/Forward.h>
 #include <wtf/RefCounted.h>
 #include <wtf/ThreadSafeRefCounted.h>
@@ -48,6 +48,10 @@
 typedef struct _GBytes GBytes;
 #endif
 
+#if USE(GSTREAMER)
+#include "GStreamerCommon.h"
+#endif
+
 #if USE(FOUNDATION)
 OBJC_CLASS NSArray;
 OBJC_CLASS NSData;
@@ -88,6 +92,9 @@ public:
     static Ref<SharedBuffer> create(GBytes*);
 #endif
 
+#if USE(GSTREAMER)
+    static Ref<SharedBuffer> create(GstMappedBuffer&);
+#endif
     // Calling data() causes all the data segments to be copied into one segment if they are not already.
     // Iterate the segments using begin() and end() instead.
     // FIXME: Audit the call sites of this function and replace them with iteration if possible.
@@ -125,6 +132,9 @@ public:
 #endif
 #if USE(GLIB)
         static Ref<DataSegment> create(GRefPtr<GBytes>&& data) { return adoptRef(*new DataSegment(WTFMove(data))); }
+#endif
+#if USE(GSTREAMER)
+        static Ref<DataSegment> create(RefPtr<GstMappedBuffer>&& data) { return adoptRef(*new DataSegment(WTFMove(data))); }
 #endif
         static Ref<DataSegment> create(FileSystem::MappedFileData&& data) { return adoptRef(*new DataSegment(WTFMove(data))); }
 
@@ -142,6 +152,10 @@ public:
 #if USE(GLIB)
         DataSegment(GRefPtr<GBytes>&& data)
             : m_immutableData(WTFMove(data)) { }
+#endif
+#if USE(GSTREAMER)
+        DataSegment(RefPtr<GstMappedBuffer>&& data)
+            : m_immutableData(WTFMove(data)) { }
 #endif
         DataSegment(FileSystem::MappedFileData&& data)
             : m_immutableData(WTFMove(data)) { }
@@ -155,6 +169,9 @@ public:
 #endif
 #if USE(GLIB)
             GRefPtr<GBytes>,
+#endif
+#if USE(GSTREAMER)
+            RefPtr<GstMappedBuffer>,
 #endif
             FileSystem::MappedFileData> m_immutableData;
         friend class SharedBuffer;
@@ -173,6 +190,9 @@ public:
 
     void hintMemoryNotNeededSoon() const;
 
+    bool operator==(const SharedBuffer&) const;
+    bool operator!=(const SharedBuffer& other) const { return !operator==(other); }
+
 private:
     explicit SharedBuffer() = default;
     explicit SharedBuffer(const char*, size_t);
@@ -188,6 +208,9 @@ private:
 #if USE(GLIB)
     explicit SharedBuffer(GBytes*);
 #endif
+#if USE(GSTREAMER)
+    explicit SharedBuffer(GstMappedBuffer&);
+#endif
 
     void combineIntoOneSegment() const;
     
@@ -202,6 +225,16 @@ private:
 #endif
 };
 
+inline bool operator==(const Ref<SharedBuffer>& left, const SharedBuffer& right)
+{
+    return left.get() == right;
+}
+
+inline bool operator!=(const Ref<SharedBuffer>& left, const SharedBuffer& right)
+{
+    return left.get() != right;
+}
+
 class WEBCORE_EXPORT SharedBufferDataView {
 public:
     SharedBufferDataView(Ref<SharedBuffer::DataSegment>&&, size_t);
