@@ -184,7 +184,20 @@ static void drawShadowLayerBuffer(PlatformContextCairo& platformContext, ImageBu
     }
 }
 
-static void fillShadowBuffer(PlatformContextCairo& platformContext, ImageBuffer& layerImage, const FloatPoint& layerOrigin, const FloatSize& layerSize, const FloatRect& sourceRect, const ShadowState& shadowState)
+// FIXME: This is mostly same as drawShadowLayerBuffer, so we should merge two.
+static void drawShadowImage(PlatformContextCairo& platformContext, ImageBuffer& layerImage, const FloatRect& destRect, const FloatRect& srcRect, const ShadowState& shadowState)
+{
+    RefPtr<Image> image = layerImage.copyImage(DontCopyBackingStore);
+    if (!image)
+        return;
+
+    if (auto surface = image->nativeImageForCurrentFrame()) {
+        drawNativeImage(platformContext, surface.get(), destRect, srcRect, shadowState.globalCompositeOperator, BlendMode::Normal, ImageOrientation(),
+            InterpolationDefault, shadowState.globalAlpha, ShadowState());
+    }
+}
+
+static void fillShadowBuffer(PlatformContextCairo& platformContext, ImageBuffer& layerImage, const FloatPoint& layerOrigin, const FloatSize& layerSize, const ShadowState& shadowState)
 {
     save(platformContext);
 
@@ -197,7 +210,7 @@ static void fillShadowBuffer(PlatformContextCairo& platformContext, ImageBuffer&
     FillSource fillSource;
     fillSource.globalAlpha = shadowState.globalAlpha;
     fillSource.color = shadowState.color;
-    fillRect(platformContext, FloatRect(layerOrigin, sourceRect.size()), fillSource, ShadowState());
+    fillRect(platformContext, FloatRect(layerOrigin, expandedIntSize(layerSize)), fillSource, ShadowState());
 
     restore(platformContext);
 }
@@ -252,7 +265,7 @@ static inline void drawPathShadow(PlatformContextCairo& platformContext, const F
                 cairo_stroke(cairoShadowContext);
             }
         },
-        [&platformContext, &shadowState, &cairoContext, &path](ImageBuffer& layerImage, const FloatPoint& layerOrigin, const FloatSize& layerSize, const FloatRect&)
+        [&platformContext, &shadowState, &cairoContext, &path](ImageBuffer& layerImage, const FloatPoint& layerOrigin, const FloatSize& layerSize)
         {
             // The original path may still be hanging around on the context and endShadowLayer
             // will take care of properly creating a path to draw the result shadow. We remove the path
@@ -266,6 +279,7 @@ static inline void drawPathShadow(PlatformContextCairo& platformContext, const F
         });
 }
 
+
 static inline void fillCurrentCairoPath(PlatformContextCairo& platformContext, const FillSource& fillSource)
 {
     cairo_t* cr = platformContext.cr();
@@ -350,7 +364,7 @@ static void drawGlyphsShadow(PlatformContextCairo& platformContext, const Shadow
         {
             drawGlyphsToContext(shadowContext.platformContext()->cr(), scaledFont, syntheticBoldOffset, glyphs);
         },
-        [&platformContext, &shadowState](ImageBuffer& layerImage, const FloatPoint& layerOrigin, const FloatSize& layerSize, const FloatRect&)
+        [&platformContext, &shadowState](ImageBuffer& layerImage, const FloatPoint& layerOrigin, const FloatSize& layerSize)
         {
             drawShadowLayerBuffer(platformContext, layerImage, layerOrigin, layerSize, shadowState);
         });
@@ -690,9 +704,17 @@ void fillRect(PlatformContextCairo& platformContext, const FloatRect& rect, cons
     if (shadowState.isVisible()) {
         ShadowBlur shadow({ shadowState.blur, shadowState.blur }, shadowState.offset, shadowState.color, shadowState.ignoreTransforms);
         shadow.drawRectShadow(State::getCTM(platformContext), State::getClipBounds(platformContext), FloatRoundedRect(rect),
-            [&platformContext, &shadowState](ImageBuffer& layerImage, const FloatPoint& layerOrigin, const FloatSize& layerSize, const FloatRect& sourceRect)
+            [&platformContext, &shadowState](ImageBuffer& layerImage, const FloatPoint& layerOrigin, const FloatSize& layerSize)
             {
-                fillShadowBuffer(platformContext, layerImage, layerOrigin, layerSize, sourceRect, shadowState);
+                fillShadowBuffer(platformContext, layerImage, layerOrigin, layerSize, shadowState);
+            },
+            [&platformContext, &shadowState](ImageBuffer& layerImage, const FloatRect& destRect, const FloatRect& srcRect)
+            {
+                drawShadowImage(platformContext, layerImage, destRect, srcRect, shadowState);
+            },
+            [&platformContext](const FloatRect& rect, const Color& color)
+            {
+                fillRectWithColor(platformContext.cr(), rect, color);
             });
     }
 
@@ -713,9 +735,17 @@ void fillRoundedRect(PlatformContextCairo& platformContext, const FloatRoundedRe
     if (shadowState.isVisible()) {
         ShadowBlur shadow({ shadowState.blur, shadowState.blur }, shadowState.offset, shadowState.color, shadowState.ignoreTransforms);
         shadow.drawRectShadow(State::getCTM(platformContext), State::getClipBounds(platformContext), rect,
-            [&platformContext, &shadowState](ImageBuffer& layerImage, const FloatPoint& layerOrigin, const FloatSize& layerSize, const FloatRect& sourceRect)
+            [&platformContext, &shadowState](ImageBuffer& layerImage, const FloatPoint& layerOrigin, const FloatSize& layerSize)
             {
-                fillShadowBuffer(platformContext, layerImage, layerOrigin, layerSize, sourceRect, shadowState);
+                fillShadowBuffer(platformContext, layerImage, layerOrigin, layerSize, shadowState);
+            },
+            [&platformContext, &shadowState](ImageBuffer& layerImage, const FloatRect& destRect, const FloatRect& srcRect)
+            {
+                drawShadowImage(platformContext, layerImage, destRect, srcRect, shadowState);
+            },
+            [&platformContext](const FloatRect& rect, const Color& color)
+            {
+                fillRectWithColor(platformContext.cr(), rect, color);
             });
     }
 
@@ -738,9 +768,25 @@ void fillRectWithRoundedHole(PlatformContextCairo& platformContext, const FloatR
     if (shadowState.isVisible()) {
         ShadowBlur shadow({ shadowState.blur, shadowState.blur }, shadowState.offset, shadowState.color, shadowState.ignoreTransforms);
         shadow.drawInsetShadow(State::getCTM(platformContext), State::getClipBounds(platformContext), rect, roundedHoleRect,
-            [&platformContext, &shadowState](ImageBuffer& layerImage, const FloatPoint& layerOrigin, const FloatSize& layerSize, const FloatRect& sourceRect)
+            [&platformContext, &shadowState](ImageBuffer& layerImage, const FloatPoint& layerOrigin, const FloatSize& layerSize)
             {
-                fillShadowBuffer(platformContext, layerImage, layerOrigin, layerSize, sourceRect, shadowState);
+                fillShadowBuffer(platformContext, layerImage, layerOrigin, layerSize, shadowState);
+            },
+            [&platformContext, &shadowState](ImageBuffer& layerImage, const FloatRect& destRect, const FloatRect& srcRect)
+            {
+                drawShadowImage(platformContext, layerImage, destRect, srcRect, shadowState);
+            },
+            [&platformContext](const FloatRect& rect, const FloatRect& holeRect, const Color& color)
+            {
+                // FIXME: We should use fillRectWithRoundedHole.
+                cairo_t* cr = platformContext.cr();
+                cairo_save(cr);
+                setSourceRGBAFromColor(cr, color);
+                cairo_set_fill_rule(cr, CAIRO_FILL_RULE_EVEN_ODD);
+                cairo_rectangle(cr, rect.x(), rect.y(), rect.width(), rect.height());
+                cairo_rectangle(cr, holeRect.x(), holeRect.y(), holeRect.width(), holeRect.height());
+                cairo_fill(cr);
+                cairo_restore(cr);
             });
     }
 
@@ -932,7 +978,7 @@ void drawSurface(PlatformContextCairo& platformContext, cairo_surface_t* surface
             {
                 drawPatternToCairoContext(shadowContext.platformContext()->cr(), pattern.get(), destRect, 1);
             },
-            [&platformContext, &shadowState](ImageBuffer& layerImage, const FloatPoint& layerOrigin, const FloatSize& layerSize, const FloatRect&)
+            [&platformContext, &shadowState](ImageBuffer& layerImage, const FloatPoint& layerOrigin, const FloatSize& layerSize)
             {
                 drawShadowLayerBuffer(platformContext, layerImage, layerOrigin, layerSize, shadowState);
             });
@@ -1205,6 +1251,14 @@ void endTransparencyLayer(PlatformContextCairo& platformContext)
     cairo_paint_with_alpha(cr, platformContext.layers().takeLast());
 }
 
+static void doClipWithAntialias(cairo_t* cr, cairo_antialias_t antialias)
+{
+    auto savedAntialiasRule = cairo_get_antialias(cr);
+    cairo_set_antialias(cr, antialias);
+    cairo_clip(cr);
+    cairo_set_antialias(cr, savedAntialiasRule);
+}
+
 void clip(PlatformContextCairo& platformContext, const FloatRect& rect)
 {
     cairo_t* cr = platformContext.cr();
@@ -1216,11 +1270,8 @@ void clip(PlatformContextCairo& platformContext, const FloatRect& rect)
     // edge fringe artifacts may occur at the layer edges
     // when a transformation is applied to the GraphicsContext
     // while drawing the transformed layer.
-    cairo_antialias_t savedAntialiasRule = cairo_get_antialias(cr);
-    cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
-    cairo_clip(cr);
+    doClipWithAntialias(cr, CAIRO_ANTIALIAS_NONE);
     cairo_set_fill_rule(cr, savedFillRule);
-    cairo_set_antialias(cr, savedAntialiasRule);
 
     if (auto* graphicsContextPrivate = platformContext.graphicsContextPrivate())
         graphicsContextPrivate->clip(rect);
@@ -1235,7 +1286,7 @@ void clipOut(PlatformContextCairo& platformContext, const FloatRect& rect)
     cairo_rectangle(cr, rect.x(), rect.y(), rect.width(), rect.height());
     cairo_fill_rule_t savedFillRule = cairo_get_fill_rule(cr);
     cairo_set_fill_rule(cr, CAIRO_FILL_RULE_EVEN_ODD);
-    cairo_clip(cr);
+    doClipWithAntialias(cr, CAIRO_ANTIALIAS_NONE);
     cairo_set_fill_rule(cr, savedFillRule);
 }
 
@@ -1249,7 +1300,8 @@ void clipOut(PlatformContextCairo& platformContext, const Path& path)
 
     cairo_fill_rule_t savedFillRule = cairo_get_fill_rule(cr);
     cairo_set_fill_rule(cr, CAIRO_FILL_RULE_EVEN_ODD);
-    cairo_clip(cr);
+    // Enforce default antialias when clipping paths, since they can contain curves.
+    doClipWithAntialias(cr, CAIRO_ANTIALIAS_DEFAULT);
     cairo_set_fill_rule(cr, savedFillRule);
 }
 
@@ -1262,7 +1314,8 @@ void clipPath(PlatformContextCairo& platformContext, const Path& path, WindRule
 
     cairo_fill_rule_t savedFillRule = cairo_get_fill_rule(cr);
     cairo_set_fill_rule(cr, clipRule == WindRule::EvenOdd ? CAIRO_FILL_RULE_EVEN_ODD : CAIRO_FILL_RULE_WINDING);
-    cairo_clip(cr);
+    // Enforce default antialias when clipping paths, since they can contain curves.
+    doClipWithAntialias(cr, CAIRO_ANTIALIAS_DEFAULT);
     cairo_set_fill_rule(cr, savedFillRule);
 
     if (auto* graphicsContextPrivate = platformContext.graphicsContextPrivate())
