@@ -30,9 +30,13 @@
 #if USE(CAIRO)
 
 #include "CairoOperations.h"
+#include "FloatRoundedRect.h"
 #include "Font.h"
 #include "GlyphBuffer.h"
 #include "GraphicsContextPlatformPrivateCairo.h"
+#include "ImageBuffer.h"
+#include "IntRect.h"
+
 
 namespace WebCore {
 
@@ -151,6 +155,8 @@ void GraphicsContextImplCairo::fillRect(const FloatRect& rect, const Color& colo
 void GraphicsContextImplCairo::fillRect(const FloatRect& rect, Gradient& gradient)
 {
     RefPtr<cairo_pattern_t> platformGradient = adoptRef(gradient.createPlatformGradient(1.0));
+    if (!platformGradient)
+        return;
 
     Cairo::save(m_platformContext);
     Cairo::fillRect(m_platformContext, rect, platformGradient.get());
@@ -242,6 +248,7 @@ void GraphicsContextImplCairo::drawGlyphs(const Font& font, const GlyphBuffer& g
         for (size_t i = 0; i < numGlyphs; ++i) {
             glyphs[i] = { glyphsData[i], xOffset, yOffset };
             xOffset += advances[i].width();
+            yOffset -= advances[i].height();
         }
     }
 
@@ -294,16 +301,15 @@ void GraphicsContextImplCairo::drawLine(const FloatPoint& point1, const FloatPoi
     Cairo::drawLine(m_platformContext, point1, point2, state.strokeStyle, state.strokeColor, state.strokeThickness, state.shouldAntialias);
 }
 
-void GraphicsContextImplCairo::drawLinesForText(const FloatPoint& point, const DashArray& widths, bool printing, bool doubleUnderlines, float strokeThickness)
+void GraphicsContextImplCairo::drawLinesForText(const FloatPoint& point, float thickness, const DashArray& widths, bool printing, bool doubleUnderlines)
 {
-    UNUSED_PARAM(strokeThickness);
     auto& state = graphicsContext().state();
-    Cairo::drawLinesForText(m_platformContext, point, widths, printing, doubleUnderlines, state.strokeColor, state.strokeThickness);
+    Cairo::drawLinesForText(m_platformContext, point, thickness, widths, printing, doubleUnderlines, state.strokeColor);
 }
 
-void GraphicsContextImplCairo::drawLineForDocumentMarker(const FloatPoint& origin, float width, DocumentMarkerLineStyle style)
+void GraphicsContextImplCairo::drawDotsForDocumentMarker(const FloatRect& rect, DocumentMarkerLineStyle style)
 {
-    Cairo::drawLineForDocumentMarker(m_platformContext, origin, width, style);
+    Cairo::drawDotsForDocumentMarker(m_platformContext, rect, style);
 }
 
 void GraphicsContextImplCairo::drawEllipse(const FloatRect& rect)
