@@ -55,7 +55,7 @@ std::unique_ptr<DisplayList> Replayer::replay(const FloatRect& initialClip, bool
     for (size_t i = 0; i < numItems; ++i) {
         auto& item = m_displayList.list()[i].get();
 
-        if (is<DrawingItem>(item)) {
+        if (!initialClip.isZero() && is<DrawingItem>(item)) {
             const DrawingItem& drawingItem = downcast<DrawingItem>(item);
             if (drawingItem.extentKnown() && !drawingItem.extent().intersects(initialClip)) {
                 LOG_WITH_STREAM(DisplayLists, stream << "skipping " << i << " " << item);
