@@ -37,40 +37,7 @@ Ref<FESpecularLighting> FESpecularLighting::create(Filter& filter, const Color&
     return adoptRef(*new FESpecularLighting(filter, lightingColor, surfaceScale, specularConstant, specularExponent, kernelUnitLengthX, kernelUnitLengthY, WTFMove(lightSource)));
 }
 
-FESpecularLighting::~FESpecularLighting()
-{
-}
-
-const Color& FESpecularLighting::lightingColor() const
-{
-    return m_lightingColor;
-}
-
-bool FESpecularLighting::setLightingColor(const Color& lightingColor)
-{
-    if (m_lightingColor == lightingColor)
-        return false;
-    m_lightingColor = lightingColor;
-    return true;
-}
-
-float FESpecularLighting::surfaceScale() const
-{
-    return m_surfaceScale;
-}
-
-bool FESpecularLighting::setSurfaceScale(float surfaceScale)
-{
-    if (m_surfaceScale == surfaceScale)
-        return false;
-    m_surfaceScale = surfaceScale;
-    return true;
-}
-
-float FESpecularLighting::specularConstant() const
-{
-    return m_specularConstant;
-}
+FESpecularLighting::~FESpecularLighting() = default;
 
 bool FESpecularLighting::setSpecularConstant(float specularConstant)
 {
@@ -80,11 +47,6 @@ bool FESpecularLighting::setSpecularConstant(float specularConstant)
     return true;
 }
 
-float FESpecularLighting::specularExponent() const
-{
-    return m_specularExponent;
-}
-
 bool FESpecularLighting::setSpecularExponent(float specularExponent)
 {
     if (m_specularExponent == specularExponent)
@@ -93,50 +55,16 @@ bool FESpecularLighting::setSpecularExponent(float specularExponent)
     return true;
 }
 
-float FESpecularLighting::kernelUnitLengthX() const
+TextStream& FESpecularLighting::externalRepresentation(TextStream& ts, RepresentationType representation) const
 {
-    return m_kernelUnitLengthX;
-}
-
-bool FESpecularLighting::setKernelUnitLengthX(float kernelUnitLengthX)
-{
-    if (m_kernelUnitLengthX == kernelUnitLengthX)
-        return false;
-    m_kernelUnitLengthX = kernelUnitLengthX;
-    return true;
-}
-
-float FESpecularLighting::kernelUnitLengthY() const
-{
-    return m_kernelUnitLengthY;
-}
-
-bool FESpecularLighting::setKernelUnitLengthY(float kernelUnitLengthY)
-{
-    if (m_kernelUnitLengthY == kernelUnitLengthY)
-        return false;
-    m_kernelUnitLengthY = kernelUnitLengthY;
-    return true;
-}
-
-const LightSource& FESpecularLighting::lightSource() const
-{
-    return m_lightSource.get();
-}
-
-void FESpecularLighting::dump()
-{
-}
-
-TextStream& FESpecularLighting::externalRepresentation(TextStream& ts, int indent) const
-{
-    writeIndent(ts, indent);
-    ts << "[feSpecularLighting";
-    FilterEffect::externalRepresentation(ts);
+    ts << indent << "[feSpecularLighting";
+    FilterEffect::externalRepresentation(ts, representation);
     ts << " surfaceScale=\"" << m_surfaceScale << "\" "
        << "specualConstant=\"" << m_specularConstant << "\" "
        << "specularExponent=\"" << m_specularExponent << "\"]\n";
-    inputEffect(0)->externalRepresentation(ts, indent + 1);
+
+    TextStream::IndentScope indentScope(ts);
+    inputEffect(0)->externalRepresentation(ts, representation);
     return ts;
 }
 
